package cz.sa.shiftPlanner.daoImpl;

import java.util.List;

import jxl.common.Logger;

import org.joda.time.DateTime;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.BusTurnsTimetableDAO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;

@Repository
public class BusTurnsTimetableDAOImpl extends AbstractDao implements BusTurnsTimetableDAO {

  private static final Logger log = Logger.getLogger(BusTurnsTimetableDAOImpl.class);
  
	@Override
	public BusTurnsTimetableVO get(Long id) {
		if(id==null) throw new IllegalArgumentException("timetable");
		return (BusTurnsTimetableVO) getSession().get(BusTurnsTimetableVO.class, id);
	}

	@Override
	public void create(BusTurnsTimetableVO timetable) {
		if ( timetable == null)
       throw new IllegalArgumentException("timetable");
    if (timetable.getId() != null)
       throw new IllegalArgumentException("Already persisted");

        getSession().saveOrUpdate(timetable);
        log.info("Turn "+timetable.getId()+" was created");
	}

	@Override
	public void update(BusTurnsTimetableVO timetable) {
		if ( timetable == null)
            throw new IllegalArgumentException("timetable");

		getSession().saveOrUpdate(timetable);
	}

	@Override
	public void remove(BusTurnsTimetableVO timetable) {
		if ( timetable == null)
            throw new IllegalArgumentException("timetable");
        if (timetable.getId() == null)
            throw new IllegalArgumentException("Already persisted");

        getSession().delete(timetable);
	}

	@Override
	public List<BusTurnsTimetableVO> findAll() {
		return getSession().createQuery("from BusTurnsTimetableVO").list();
	}

	public List<BusTurnsTimetableVO> findByLineGroupId(Long id) {
		if(id==null) throw new IllegalArgumentException("timetable");
		return getSession().createQuery("SELECT t from BusTurnsTimetableVO AS t where t.line.id = :line order by t.coupleNumber , t.departureTime").setParameter("line", id).list();
	}


	@Override
	public List<BusTurnsTimetableVO> findByLineGroupIdAndDay(Long id, WeekDay day, Integer week, DateTime validity) {
		return getSession().createQuery(
				"SELECT turn from BusTurnsTimetableVO AS turn" +
				" left join turn.rules as rule" +
				" left join turn.timetable AS time" +
				" where time.line.id = :line AND turn.weekDay = :day AND turn.week =:week" +
				" AND (time.validity_from < :validity AND time.validity_to > :validity)"+
				" order by turn.coupleNumber , time.departureTime")
				.setParameter("line", id)
				.setParameter("week", week)
				.setParameter("validity", validity)
				.setParameter("day", day).list();
	}

	@Override
	public BusTurnsTimetableVO findByDayAndTimetable(BusTimetableVO timetable, WeekDay day, Integer week) {
		WeekDay dayLoc = day;
		if(dayLoc.equals(WeekDay.WHOLEWEEK) || dayLoc.equals(WeekDay.WORKDAY)) dayLoc = WeekDay.MONDAY;
		return (BusTurnsTimetableVO) getSession().createQuery(
				"SELECT t FROM BusTurnsTimetableVO as t" +
				" where t.weekDay = :day AND t.timetable.id = :timetable" +
				" AND t.validity_from is null AND t.week = :week")
				.setParameter("day", dayLoc)
				.setParameter("timetable", timetable.getId())
				.setParameter("week", week)
				.setMaxResults(1).uniqueResult();
	}


	@Override
	public BusTurnsTimetableVO findByDayAndTimetable(BusTimetableVO timetable, WeekDay day, DateTime validity, Integer week) {
	  Long count = 0L;
		//zjistim jestli existuje pro dany den nejaka vyjimka (count vrati nenulovou hodnotu). Pokud existuje vyjimka pak hledam jen mezi vyjimkama.
	  if(validity != null){
		  count = (Long) getSession().createQuery(
				"SELECT count(t) FROM BusTurnsTimetableVO as t" +
						" where t.weekDay = :day" +
				" AND (t.validity_from <= :validity AND t.validity_to >= :validity)")
				.setParameter("day", day)
				.setParameter("validity", validity).uniqueResult();
	  }
		if(count == 0){
			return (BusTurnsTimetableVO) getSession().createQuery(
					"SELECT t FROM BusTurnsTimetableVO as t" +
					" where t.weekDay = :day AND t.timetable.id = :timetable AND t.week = :week" +
					" AND ( t.validity_from is null)" +//(t.validity_from <= :validity AND t.validity_to >= :validity) OR
					" order by t.validity_from")
					.setParameter("day", day)
					.setParameter("week", week)
					//.setParameter("validity", validity)
					.setParameter("timetable", timetable.getId())
					.setMaxResults(1).uniqueResult();
		}else{
			return (BusTurnsTimetableVO) getSession().createQuery(
					"SELECT t FROM BusTurnsTimetableVO as t" +
					" where t.weekDay = :day AND t.timetable.id = :timetable" +
					" AND ((t.validity_from <= :validity AND t.validity_to >= :validity))" +
					" order by t.validity_from")
					.setParameter("day", day)
					.setParameter("validity", validity)
					.setParameter("timetable", timetable.getId())
					.setMaxResults(1).uniqueResult();
		}
	}

	@Override
	public BusTurnsTimetableVO findByDayAndTimetableOB(BusTimetableVO timetable, WeekDay day) {

		return (BusTurnsTimetableVO) getSession().createQuery(
				"SELECT t FROM BusTurnsTimetableVO as t" +
				" where t.weekDay = :day AND t.timetable.id = :timetable" +
				" order by t.validity_from")
				.setParameter("day", day)
				.setParameter("timetable", timetable.getId())
				.setMaxResults(1).uniqueResult();
	}

	@Override
	public List<BusTurnsTimetableVO> findByTimetable(Long timId) {
		
		return  getSession().createQuery(
				"SELECT t FROM BusTurnsTimetableVO as t" +
				" where  t.timetable.id = :timetable")
				.setParameter("timetable", timId)
				.list();
	}
	
}
