package cz.sa.shiftPlanner.daoImpl;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.criterion.Order;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.CityDAO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.StationVO;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;

@Repository
public class CityDAOImpl extends AbstractDao implements CityDAO {

  private static final Logger log = Logger.getLogger(CityDAOImpl.class);
  
  
  @Override
  public CityVO get(Long id) {
    if (id == null) //throw new IllegalArgumentException("city");
      return null;
    Object o = getSession().get(CityVO.class, id);
    return (CityVO) o;
  }

  @Override
  public void create(CityVO city) {
    if (city == null)
      throw new IllegalArgumentException("city");
    if (city.getId() == null)
      throw new IllegalArgumentException("In this entity must be ID manually set befor call save()");

    getSession().save(city);
    log.info("city saved");
  }

  @Override
  public void update(CityVO city) {
    if (city == null)
      throw new IllegalArgumentException("city");
    if (city.getId() == null)
      throw new IllegalArgumentException("id is null");

    getSession().merge(city);
    log.info("city updated");
  }

  @Override
  public void remove(CityVO city) {
    if (city == null)
      throw new IllegalArgumentException("city");
    if (city.getId() == null)
      throw new IllegalArgumentException("id is null");

    getSession().delete(city);

  }

  @Override
  public List<CityVO> findAll() {
    return getSession().createQuery("FROM CityVO order by cityName").list();
  }

  @Override
  public List<CityVO> findByName(String name) {
    return getSession().createQuery("SELECT c FROM CityVO as c WHERE c.name= :name").setParameter("name", name).list();
  }

  @Override
  public void save(CityVO c) {
    getSession().save(c);
  }

  @Override
  public List<CityVO> findByCityId(Long id) {
    if (id == null)
      return new ArrayList<CityVO>();
    return getSession().createQuery("SELECT c FROM CityVO as c WHERE c.cityId= :id").setParameter("id", id).list();
  }

  @Override
  public void createStation(StationVO station) {
    if (station == null)
      throw new IllegalArgumentException("station");

    getSession().saveOrUpdate(station);
    log.info("Station saved or updated");
  }

  @Override
  public StationVO getStation(Long stationId) {
    if (stationId == null)
      throw new IllegalArgumentException("city");
    return (StationVO) getSession().get(StationVO.class, stationId);
   
  }

  @Override
  public List<CityVO> findAllCZ() {
    return getSession().createQuery("SELECT c FROM CityVO c where country = 'CZ' order by cityName").list();
  }

  @Override
  public List<StationVO> findAllStations() {
    return getSession().createCriteria(StationVO.class).addOrder(Order.asc("city")).list();
  }

}
