package cz.sa.shiftPlanner.daoImpl;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.joda.time.DateTime;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.PersonUtilsDAO;
import cz.sa.shiftPlanner.entities.ExcuseVO;
import cz.sa.shiftPlanner.entities.PersonRecordVO;
import cz.sa.shiftPlanner.entities.PersonVO;
import cz.sa.shiftPlanner.entities.TimeTrialVO;
import cz.sa.shiftPlanner.entities.WorkHoursItemVO;
import cz.sa.shiftPlanner.entities.WorkHoursVO;
import cz.sa.shiftPlanner.enums.ExcuseReason;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.TimeRange;

/**
 * @author ToSk
 */

public class PersonUtilsDAOImpl extends AbstractDao implements PersonUtilsDAO {

  @Override
  public void createExcuse(ExcuseVO excuse) {
    if (excuse == null)
      throw new IllegalArgumentException("excuse");
    if (excuse.getId() != null)
      throw new IllegalArgumentException("Already persisted");

    getSession().save(excuse);
  }

  @Override
  public List<ExcuseVO> findAll() {

    return getSession().createQuery("select e from ExcuseVO as e").list();
  }

  @Override
  public List<ExcuseVO> findAllByPersonId(Long id) {

    return getSession().createQuery("select e from ExcuseVO as e where e.person.id = :id order by e.from_ desc")
        .setParameter("id", id).list();
  }

  @Override
  public void createWorkHours(WorkHoursVO hours) {

    if (hours == null)
      throw new IllegalArgumentException("hours");
    if (hours.getId() != null)
      throw new IllegalArgumentException("Already persisted");

    getSession().save(hours);
  }

  @Override
  public WorkHoursVO findHourByIdAndMonth(Long personId, int month, int year) {

    return (WorkHoursVO) getSession()
        .createQuery(
            "select w from WorkHoursVO as w left join w.items where w.person.id = :id and w.month = :month and w.year = :year")
        .setParameter("id", personId).setParameter("month", month).setParameter("year", year).uniqueResult();
  }

  @Override
  public void createWorkHoursItem(WorkHoursItemVO hours) {

    if (hours == null)
      throw new IllegalArgumentException("hours");
    if (hours.getId() != null)
      throw new IllegalArgumentException("Already persisted");

    getSession().save(hours);
    WorkHoursVO h = getHours(hours.getWorkHours());
    h.setMonthAmount(sumMonthHours(h.getId()).intValue());
    updateWorkHours(h);
  }

  @Override
  public WorkHoursVO getHours(WorkHoursVO hours) {
    return (WorkHoursVO) getSession().get(WorkHoursVO.class, hours.getId());
  }

  @Override
  public void updateWorkHours(WorkHoursVO hours) {
    getSession().update(hours);
  }

  @Override
  public void updateWorkHoursItem(WorkHoursItemVO hours) {

    getSession().update(hours);
    WorkHoursVO h = getHours(hours.getWorkHours());
    h.setMonthAmount(sumMonthHours(h.getId()).intValue());
    updateWorkHours(h);
  }

  @Override
  public WorkHoursItemVO findItemByIdAndDay(Long HoursId, int day) {

    return (WorkHoursItemVO) getSession()
        .createQuery("select w from WorkHoursItemVO as w where w.workHours.id = :id and w.dayOfMonth = :dayOfMonth")
        .setParameter("id", HoursId).setParameter("dayOfMonth", day).uniqueResult();
  }

  @Override
  public Long sumMonthHours(Long hoursId) {

    return (Long) getSession().createQuery("select sum(w.day_amount) from WorkHoursItemVO as w where w.workHours.id = :id")
        .setParameter("id", hoursId).uniqueResult();
  }

  @Override
  public List<TimeRange> rangesInDay(DateTime dt, PersonVO p) {

    DateTime from = new DateTime(dt.getYear(), dt.getMonthOfYear(), dt.getDayOfMonth(), 0, 0, 0, 0);
    DateTime to = new DateTime(dt.getYear(), dt.getMonthOfYear(), dt.getDayOfMonth(), 23, 59, 59, 0);
    List<ExcuseVO> list = getSession()
        .createQuery("select e from ExcuseVO as e where e.person.id = :id AND" + " (e.from_ <= :from AND e.to_ >= :to) OR" + //interval casovky z obou stran presahuje interval from-to
            " (e.from_ >= :from AND e.to_ <= :to)  OR" + //interval casovky je cely vnoreny v intervalu from-to
            " (e.from_ <= :from AND e.to_ <= :to AND e.to_ >= :from) OR" + //interval casovky presahuje jen spodni hranici intervalu from-to
            " (e.from_ >= :from AND e.to_ >= :to AND e.from_ <= :to)") //interval casovky presahuje jen horni hranici intervalu from-to
        .setParameter("from", from).setParameter("to", to).setParameter("id", p.getId()).list();

    List<TimeRange> result = new ArrayList<TimeRange>();
    for (ExcuseVO e : list) {
      TimeRange t = new TimeRange(e.getFrom_(), e.getTo_());
      result.add(t);
    }
    return result;
  }

  @Override
  public List<ExcuseVO> excusesInDay(DateTime dt, ExcuseReason reason) {

    DateTime from = new DateTime(dt.getYear(), dt.getMonthOfYear(), dt.getDayOfMonth(), 0, 0, 0, 0);
    DateTime to = new DateTime(dt.getYear(), dt.getMonthOfYear(), dt.getDayOfMonth(), 23, 59, 59, 0);
    String query = "select e from ExcuseVO as e where " + " ((e.from_ <= :from AND e.to_ >= :to) OR" + //interval casovky z obou stran presahuje interval from-to
        " (e.from_ >= :from AND e.to_ <= :to)  OR" + //interval casovky je cely vnoreny v intervalu from-to
        " (e.from_ <= :from AND e.to_ <= :to AND e.to_ >= :from) OR" + //interval casovky presahuje jen spodni hranici intervalu from-to
        " (e.from_ >= :from AND e.to_ >= :to AND e.from_ <= :to))"; //interval casovky presahuje jen horni hranici intervalu from-to
    query += " AND e.status = 'APPROVED'";
    if (reason != null)
      query += " AND e.reason = :reason";
    Query q = getSession().createQuery(query);
    if (reason != null)
      q.setParameter("reason", reason);
    q.setParameter("from", from);
    q.setParameter("to", to);
    return q.list();
  }

  @Override
  public void remove(ExcuseVO ex) {
    getSession().delete(ex);
  }

  @Override
  public void update(ExcuseVO ex) {
    getSession().merge(ex);
  }

  @Override
  public List<TimeTrialVO> findAllByPersonId(long personId, DateTime from, DateTime to) {

    return getSession()
        .createQuery(
            "select t from TimeTrialVO as t" + " where t.person.id = :id"
                + " AND ((t.from_ >= :from AND t.from_ <= :to) OR (t.to_ >= :from AND t.to_ <= :to))"
                + " order by t.from_ desc").setParameter("from", from).setParameter("to", to).setParameter("id", personId)
        .list();
  }

  @Override
  public void create(TimeTrialVO tt) {
    if (tt == null)
      throw new IllegalArgumentException("time trial");
    if (tt.getId() != null)
      throw new IllegalArgumentException("Already persisted");
    getSession().save(tt);
  }

  @Override
  public void update(TimeTrialVO tt) {
    if (tt == null)
      throw new IllegalArgumentException("time trial");
    if (tt.getId() == null)
      throw new IllegalArgumentException("Already persisted");
    getSession().update(tt);
  }

  @Override
  public void remove(TimeTrialVO tt) {
    if (tt == null)
      throw new IllegalArgumentException("time trial");
    if (tt.getId() == null)
      throw new IllegalArgumentException("Already persisted");
    getSession().delete(tt);
  }

  @Override
  public List<TimeTrialVO> findAllByDay(DateTime day) {

    DateTime from = DateFormatter.toBeginOfDay(day);
    DateTime to = DateFormatter.toEndOfDay(day);

    return getSession()
        .createQuery("select distinct t from TimeTrialVO as t" + " WHERE (t.from_ <= :from AND t.to_ >= :to) OR" + //interval casovky z obou stran presahuje interval from-to
            " (t.from_ >= :from AND t.to_ <= :to)  OR" + //interval casovky je cely vnoreny v intervalu from-to
            " (t.from_ <= :from AND t.to_ <= :to AND t.to_ >= :from) OR" + //interval casovky presahuje jen spodni hranici intervalu from-to
            " (t.from_ >= :from AND t.to_ >= :to AND t.from_ <= :to)") //interval casovky presahuje jen horni hranici intervalu from-to
        .setParameter("from", from).setParameter("to", to).list();
  }

  @Override
  public void createRecord(PersonRecordVO rec) {
    if (rec == null)
      throw new IllegalArgumentException("record");
    if (rec.getId() != null)
      throw new IllegalArgumentException("Already persisted");
    getSession().save(rec);
  }

  @Override
  public List<PersonRecordVO> findRecordByPerson(PersonVO p) {
    return getSession().createCriteria(PersonRecordVO.class).add(Restrictions.eq("person", p))
        .addOrder(Order.desc("dateCreated")).list();
  }

  @Override
  public void removeRecord(PersonRecordVO rec) {
    if (rec == null)
      throw new IllegalArgumentException("record");
    if (rec.getId() == null)
      throw new IllegalArgumentException("Already persisted");
    getSession().delete(rec);
  }

}
