package cz.sa.shiftPlanner.daoImpl;

import java.util.List;

import jxl.common.Logger;

import org.joda.time.DateTime;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.RuleDAO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.RuleVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;

@Repository
public class RuleDAOImpl extends AbstractDao implements RuleDAO {

  private static final Logger log = Logger.getLogger(RuleDAOImpl.class);
      
  @Override
  public RuleVO get(Long id) {
    if (id == null)
      throw new IllegalArgumentException("rule");
    return (RuleVO) getSession().get(RuleVO.class, id);
  }

  @Override
  public void create(RuleVO rule) {
    if (rule == null)
      throw new IllegalArgumentException("rule");
    if (rule.getId() != null)
      throw new IllegalArgumentException("Already persisted");

    getSession().save(rule);
    log.info("Rule "+rule.getId()+" was created.");
  }

  @Override
  public void update(RuleVO rule) {
    if (rule == null)
      throw new IllegalArgumentException("rule");
    if (rule.getId() == null)
      throw new IllegalArgumentException("Already persisted");

    getSession().merge(rule);

  }

  @Override
  public void remove(RuleVO rule) {
    if (rule == null)
      throw new IllegalArgumentException("rule");
    if (rule.getId() == null)
      throw new IllegalArgumentException("Already persisted");

    getSession().delete(rule);

  }

  @Override
  public List<RuleVO> findByWeekAndDay(WeekDay day, Integer week, DateTime validity) {
    return getSession().createQuery("SELECT r FROM RuleVO as r" + 
                                    " join r.turn as turn" + 
                                    " join turn.timetable as t" + 
                                    " where r.week = :week AND r.turn.weekDay= :day" +
                                    " AND (t.validity_from < :validity AND t.validity_to > :validity)")
                                    .setParameter("day", day).setParameter("week", week)
                                    .setParameter("validity", validity).list();
  }

  @Override
  public RuleVO findByWeekAndTurn(BusTurnsTimetableVO turn, Integer week) {
    return (RuleVO) getSession().createQuery("SELECT r FROM RuleVO as r" + " where r.week = :week AND r.turn.id = :turn")
        .setParameter("week", week).setParameter("turn", turn.getId()).uniqueResult();
  }

  @Override
  public List<RuleVO> findByTurn(BusTurnsTimetableVO turn) {
    return getSession().createQuery("SELECT r FROM RuleVO as r where r.turn.id = :turn")
        .setParameter("turn", turn.getId()).list();
  }

}
