package cz.sa.shiftPlanner.daoImpl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.joda.time.DateTime;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.WayDAO;
import cz.sa.shiftPlanner.entities.PersonVO;
import cz.sa.shiftPlanner.entities.WayVO;
import cz.sa.shiftPlanner.entities.WayVOAudit;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;
import cz.sa.shiftPlanner.utils.DateFormatter;

@Repository
public class WayDAOImpl extends AbstractDao implements WayDAO {

	@Override
	public WayVO get(Long id) {
		if(id == null) throw new IllegalArgumentException("way");
		return (WayVO) getSession().get(WayVO.class, id);
	}
	

	@Override
	public void create(WayVO way) {
		if (way == null)
            throw new IllegalArgumentException("way");
        if (way.getId() != null)
            throw new IllegalArgumentException("Already persisted");

        getSession().save(way);
	}
	
	@Override
	public void createAuditVO(WayVOAudit auditVO) {
	  if (auditVO == null)
	    throw new IllegalArgumentException("way");
	  if (auditVO.getId() != null)
	    throw new IllegalArgumentException("Already persisted");
	  auditVO.setDateCreated(new DateTime());
	  getSession().save(auditVO);
	}

	@Override
	public void update(WayVO way) {
		if (way == null)
            throw new IllegalArgumentException("way");
        if (way.getId() == null)
            throw new IllegalArgumentException("Already persisted");

        getSession().merge(way);

	}

  @Override
  public void remove(WayVO way) {
    if (way == null)
      throw new IllegalArgumentException("way");
    if (way.getId() == null)
      throw new IllegalArgumentException("Already persisted");

    getSession().delete(way);
  }
	

	@Override
	public List<WayVO> findAll() {
		return getSession().createQuery("FROM WayVO").list();
	}

	@Override
	public List<WayVO> findByDepRange(DateTime dep, DateTime arr, PersonVO p) {

		return getSession().createQuery(
				"SELECT w from WayVO AS w" +
				" where w.person.id = :p" +
				" AND ((w.departureTime >= :from AND w.departureTime <= :to)" +
				" OR (w.arrivalTime >= :from AND w.arrivalTime <= :to))")
				.setParameter("from", dep)
				.setParameter("to", arr)
				.setParameter("p", p.getId())
				.list();
	}
	
	@Override
	public List<WayVO> findByTimeRange(DateTime from, DateTime to, PersonVO p) {

		return getSession().createQuery(
				"SELECT w from WayVO AS w" +
				" where w.person.id = :p" +
				" AND (w.departureTime >= :from AND w.departureTime <= :to)")
				.setParameter("from", from)
				.setParameter("to", to)
				.setParameter("p", p.getId())
				.list();
	}

	@Override
	public List<WayVO> findByMonth(DateTime from, DateTime to, PersonVO p) {

		List<WeekDay> days = Arrays.asList(WeekDay.values());
		int day = from.getDayOfWeek();
		DateTime _from = new DateTime(from.getYear(), from.getMonthOfYear(), from.getDayOfMonth(), 0, 0, 0, 0);
		DateTime _to = new DateTime(to.getYear(), to.getMonthOfYear(), to.getDayOfMonth(), 0, 0, 0, 0).plusDays(1);
		List<WayVO> result = new ArrayList<WayVO>();
		List<WayVO> part = new ArrayList<WayVO>();
		Integer week = DateFormatter.getWeekMod2(from);
		
		//projdu postupne vsechny dny v mesici a pro kazdy se zeptam na vysledek :-(
		while (_from.compareTo(_to) < 0) {
			//zjistim jestli existuje pro dany den nejaka vyjimka (count vrati nenulovou hodnotu). Pokud ano pak hledam jen mezi vyjimkama.
			Long count = (Long) getSession().createQuery(
					"SELECT count(t) FROM BusTurnsTimetableVO as t" +
					" where t.weekDay = :day" +
					" AND (t.validity_from <= :validity AND t.validity_to >= :validity)")
					.setParameter("day", days.get(day-1))
					.setParameter("validity", _from)
					.uniqueResult();
			if(count != 0){
				part = (List<WayVO>) getSession().createQuery(
						"SELECT w from WayVO AS w" +
						" join w.connection as c" +
						" join c.timetable as timetable" +
						" join timetable.turns as turns" +
						" where w.person.id = :p" +
						" AND (w.departureTime >= :from AND w.departureTime <= :to)" +
						" AND (turns.validity_from <= :validity AND turns.validity_to >= :validity)" +
						" order by timetable.line, turns.coupleNumber, c.departureTime")
						.setParameter("from", _from)
						.setParameter("to", _from.plusDays(1))
						.setParameter("p", p.getId())
						.setParameter("validity", _from)
						.list();
			}else{
				part = (List<WayVO>) getSession().createQuery(
						"SELECT w from WayVO AS w" +
						" join w.connection as c" +
						" join c.timetable as timetable" +
						" join timetable.turns as turns" +
						" where w.person.id = :p" +
						" AND (w.departureTime >= :from AND w.departureTime <= :to)" +
						" AND turns.validity_from is null" +
						" AND turns.weekDay = :weekday"+
						" AND turns.week = :week" +
						" order by timetable.line, turns.coupleNumber, c.departureTime")
						.setParameter("from", _from)
						.setParameter("weekday", WeekDay.getDay(_from.getDayOfWeek()))
						.setParameter("to", _from.plusDays(1))
						.setParameter("p", p.getId())
						.setParameter("week", week)
						.list();
			}
			result.addAll(part);
			_from = _from.plusDays(1);
		}
		return result;
	}
	
	@Override
  public List<WayVOAudit> findByMonthNonActive(DateTime from, DateTime to, PersonVO p) {

    List<WeekDay> days = Arrays.asList(WeekDay.values());
    int day = from.getDayOfWeek();
    DateTime _from = new DateTime(from.getYear(), from.getMonthOfYear(), from.getDayOfMonth(), 0, 0, 0, 0);
    DateTime _to = new DateTime(to.getYear(), to.getMonthOfYear(), to.getDayOfMonth(), 0, 0, 0, 0).plusDays(1);
    List<WayVOAudit> result = new ArrayList<WayVOAudit>();
    List<WayVOAudit> part = new ArrayList<WayVOAudit>();
    Integer week = DateFormatter.getWeekMod2(from);
    
    //projdu postupne vsechny dny v mesici a pro kazdy se zeptam na vysledek :-(
    while (_from.compareTo(_to) < 0) {
      //zjistim jestli existuje pro dany den nejaka vyjimka (count vrati nenulovou hodnotu). Pokud ano pak hledam jen mezi vyjimkama.
      Long count = (Long) getSession().createQuery(
          "SELECT count(t) FROM BusTurnsTimetableVO as t" +
          " where t.weekDay = :day" +
          " AND (t.validity_from <= :validity AND t.validity_to >= :validity)")
          .setParameter("day", days.get(day-1))
          .setParameter("validity", _from)
          .uniqueResult();
      if(count != 0){
        part = getSession().createSQLQuery(
            " SELECT w from WayVOAudit AS w" +
                " join w.connection as c" +
                " join c.timetable as timetable" +
                " join timetable.turns as turns" +
                " where w.person.id = :p" +
                " AND (w.departureTime >= :from AND w.departureTime <= :to)" +
                " AND (turns.validity_from <= :validity AND turns.validity_to >= :validity)" +
                " AND turns.week = :week" +
                " AND w.dateCreated in (SELECT max(way.dateCreated) from WayVOAudit as way " +
                      " WHERE  (w.departureTime >= :from AND w.departureTime <= :to)" +
                      " group by way.connection) "+
                " order by timetable.line, turns.coupleNumber, c.departureTime ")
            .setParameter("from", _from)
            .setParameter("to", _from.plusDays(1))
            .setParameter("p", p.getId())
            .setParameter("week", week)
            .list();
      }else{
        part = getSession().createQuery(
                " SELECT w from WayVOAudit AS w" +
                " join w.connection as c" +
                " join c.timetable as timetable" +
                " join timetable.turns as turns" +
                " where w.person.id = :p" +
                " AND (w.departureTime >= :from AND w.departureTime <= :to)" +
                " AND (turns.validity_from is null)" +
                " AND turns.week = :week" +
                " AND w.dateCreated in (SELECT max(way.dateCreated) from WayVOAudit as way " +
                      " WHERE  (w.departureTime >= :from AND w.departureTime <= :to)" +
                      " group by way.connection) "+
                " order by timetable.line, turns.coupleNumber, c.departureTime ")
            .setParameter("from", _from)
            .setParameter("to", _from.plusDays(1))
            .setParameter("p", p.getId())
            .setParameter("week", week)
            .list();
      }
      result.addAll(part);
      _from = _from.plusDays(1);
    }
    return result;
  }


}
