package cz.sa.shiftPlanner.entities;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.Type;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.entities.infrastructure.AbstractNoGenVO;

@Entity
@Table(name="bus_timetable")
public class BusTimetableVO extends AbstractNoGenVO {

	@ManyToOne(cascade = CascadeType.DETACH)
	private StationVO arrivalStation;
	
	@ManyToOne(cascade = CascadeType.DETACH)
	private StationVO departureStation;
	
	@ManyToOne(cascade = CascadeType.DETACH)
	@JoinColumn(name="line_group")
	private LineGroupVO line;
	
	@Type(type = "org.jadira.usertype.dateandtime.joda.PersistentDateTime")
	@Column(nullable=false)
	private DateTime arrivalTime;
	
	@Type(type = "org.jadira.usertype.dateandtime.joda.PersistentDateTime")
	@Column(nullable=false)
	private DateTime departureTime;
	
	@OneToMany( mappedBy = "timetable", cascade = CascadeType.ALL, fetch=FetchType.EAGER)
	private List<BusTurnsTimetableVO> turns;
	
	@ManyToMany(fetch=FetchType.LAZY)
	private Set<SymbolVO> symbol = new HashSet<SymbolVO>();
	
	@Column(nullable=false)
	private Integer number;

	@Type(type = "org.jadira.usertype.dateandtime.joda.PersistentDateTime")
	private DateTime validity_from;

	@Type(type = "org.jadira.usertype.dateandtime.joda.PersistentDateTime")
	private DateTime validity_to;
	
	@ManyToOne(fetch=FetchType.LAZY)
	private TimeCodeVO timeCode;
	
	
	public Set<SymbolVO> getSymbol() {
		return symbol;
	}

	public void setSymbol(Set<SymbolVO> symbol) {
		this.symbol = symbol;
	}

	public TimeCodeVO getTimeCode() {
		return timeCode;
	}

	public void setTimeCode(TimeCodeVO timeCode) {
		this.timeCode = timeCode;
	}

	public DateTime getValidity_from() {
		return validity_from;
	}

	public void setValidity_from(DateTime validity_from) {
		this.validity_from = validity_from;
	}

	public DateTime getValidity_to() {
		return validity_to;
	}

	public void setValidity_to(DateTime validity_to) {
		this.validity_to = validity_to;
	}

	public List<BusTurnsTimetableVO> getTurns() {
		return turns;
	}

	public void setTurns(List<BusTurnsTimetableVO> turns) {
		this.turns = turns;
	}

	public StationVO getArrivalStation() {
    return arrivalStation;
  }

  public void setArrivalStation(StationVO arrivalStation) {
    this.arrivalStation = arrivalStation;
  }

  public StationVO getDepartureStation() {
    return departureStation;
  }

  public void setDepartureStation(StationVO departureStation) {
    this.departureStation = departureStation;
  }

  public LineGroupVO getLine() {
		return line;
	}

	public void setLine(LineGroupVO line) {
		this.line = line;
	}

	public DateTime getArrivalTime() {
		return arrivalTime;
	}

	public void setArrivalTime(DateTime arrivalTime) {
		this.arrivalTime = arrivalTime;
	}

	public DateTime getDepartureTime() {
		return departureTime;
	}

	public void setDepartureTime(DateTime departureTime) {
		this.departureTime = departureTime;
	}

	public Integer getNumber() {
		return number;
	}

	public void setNumber(Integer number) {
		this.number = number;
	}

	public Set<SymbolVO> getSymbols() {
		return symbol;
	}

	public void setSymbols(Set<SymbolVO> symbols) {
		this.symbol = symbols;
	}

	
}
