package cz.sa.shiftPlanner.entities;

import java.util.Date;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;

import org.joda.time.LocalDate;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;
import com.ibm.icu.util.Holiday;

import cz.sa.shiftPlanner.entities.infrastructure.AbstractVO;
import cz.sa.shiftPlanner.utils.MySimpleHoliday;



@Entity
@Table(name="symbol")
public class SymbolVO extends AbstractVO{

	public static final String MONDAY = "1";
	  public static final String TUESDAY = "2";
	  public static final String WEDNESDAY = "3";
	  public static final String THURSDAY = "4";
	  public static final String FRIDAY = "5";
	  public static final String SATURDAY = "6";
	  public static final String SUNDAY = "7";
	  public static final String WORKDAY = "x";
	  public static final String HOLIDAY = "+";
	  
	//http://en.wikipedia.org/wiki/Public_holidays_in_the_Czech_Republic
	  public static Set<MySimpleHoliday> czechHoliday = Sets.newHashSet(
	      // fuck! pouziva java.util.date, takze mesice jsou indexovane od 0!!
		  new MySimpleHoliday(0, 1, "Nový rok"),
		  new MySimpleHoliday(3, 1, "Velikonoční pondělí"),
	      new MySimpleHoliday(4, 1, "Svátek práce"), // Labour day
	      new MySimpleHoliday(4, 8, "Den osvobození"), //Den vítězství or Liberation day
	      new MySimpleHoliday(6, 5, "Den slovanských věrozvěstů Cyrila a Metoděje"), //Saints Cyril and Methodius Day
	      new MySimpleHoliday(6, 6, "Den upálení mistra Jana Husa"), //Jan Hus Day
	      new MySimpleHoliday(8, 28, "Den české státnosti"), //St. Wenceslas Day (Czech Statehood Day)
	      new MySimpleHoliday(9, 28, "Den vzniku samostatného československého státu"), //Independent Czechoslovak State Day
	      new MySimpleHoliday(10, 17, "Den boje za svobodu a demokracii"), //Struggle for Freedom and Democracy Day
	      new MySimpleHoliday(11, 24, "Štědrý den"),
	      new MySimpleHoliday(11, 25, "1. svátek vánoční (Boží hod)"),
	      new MySimpleHoliday(11, 26, "2. svátek vánoční")
	      );
	  
	  @Column
	  private String mark;

	  @Column
	  private String description;


	public String getMark() {
		return mark;
	}

	public void setMark(String mark) {
		this.mark = mark;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}	  
	  
	 public static boolean isHoliday(final LocalDate localDate) {
		
		 final Date date = localDate.toDateTimeAtStartOfDay().toDate();
		 
		 return Iterables.any(czechHoliday, new Predicate<Holiday>() {
			 public boolean apply(Holiday holiday) {
				 return holiday.isOn(date);
			 }
		 });
	 }
}
