package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dozer.Mapper;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.BusTimetableDAO;
import cz.sa.shiftPlanner.dao.BusTurnsTimetableDAO;
import cz.sa.shiftPlanner.dao.CityDAO;
import cz.sa.shiftPlanner.dao.LineGroupDAO;
import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.TimeCodeItemDTO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.BusTimetableService;
import cz.sa.shiftPlanner.utils.TimeRange;
import cz.sa.shiftPlanner.utils.Tuple;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class BusTimetableServiceImpl implements BusTimetableService {

	private CityDAO cityDAO;
	private LineGroupDAO lineGroupDAO;
	private BusTimetableDAO busTimetableDAO;
	private BusTurnsTimetableDAO busTurnsTimetableDAO;

	private Mapper mapper; 
	
	public Mapper getMapper() {
		return mapper;
	}

	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}
	public CityDAO getCityDAO() {
		return cityDAO;
	}

	
	public BusTurnsTimetableDAO getBusTurnsTimetableDAO() {
		return busTurnsTimetableDAO;
	}

	public void setBusTurnsTimetableDAO(BusTurnsTimetableDAO busTurnsTimetableDAO) {
		this.busTurnsTimetableDAO = busTurnsTimetableDAO;
	}

	public void setCityDAO(CityDAO cityDAO) {
		this.cityDAO = cityDAO;
	}

	public LineGroupDAO getLineGroupDAO() {
		return lineGroupDAO;
	}

	public void setLineGroupDAO(LineGroupDAO lineGroupDAO) {
		this.lineGroupDAO = lineGroupDAO;
	}

	public BusTimetableDAO getBusTimetableDAO() {
		return busTimetableDAO;
	}

	public void setBusTimetableDAO(BusTimetableDAO busTimetableDAO) {
		this.busTimetableDAO = busTimetableDAO;
	}

	

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void create(BusTimetableDTO timetable) {

		if (timetable == null)
			throw new IllegalArgumentException("timetable is null");
		BusTimetableVO b = mapper.map(timetable, BusTimetableVO.class);

		busTimetableDAO.create(b);
		mapper.map(b,timetable);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void update(BusTimetableDTO timetable) {

		if (timetable == null)
			throw new IllegalArgumentException("timetable is null");

		
		BusTimetableVO vo = mapper.map(timetable, BusTimetableVO.class);
		
		busTimetableDAO.update(vo);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void remove(BusTimetableDTO timetable) {
		busTimetableDAO.remove(busTimetableDAO.get(timetable.getId()));
	}

	
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public Map<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>> findByLineGroupId(Long id, DateTime validity) {
	
		List<BusTimetableDTO> list = new ArrayList<BusTimetableDTO>();
		for(BusTimetableVO p : busTimetableDAO .findByLineGroupId(id, validity)){
			list.add(mapper.map(p, BusTimetableDTO.class));
		}
		return sortListToMap(list);
	}
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public Map<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>> findByLineGroupIdAndDay(Long id, WeekDay day, DateTime validity) {

		List<BusTimetableDTO> list = new ArrayList<BusTimetableDTO>();
		for(BusTimetableVO p : busTimetableDAO .findByLineGroupIdAndDay(id, day, validity)){
			list.add(mapper.map(p, BusTimetableDTO.class));
			
		}
		return sortListToMap(list);
	}
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public Map<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>> findByLineGroupIdAndDay(Long id, DateTime day, DateTime validity) {

		List<BusTimetableDTO> list = new ArrayList<BusTimetableDTO>();
		List<BusTimetableVO> times = busTimetableDAO .findByLineGroupIdAndDay(id, day, validity);
		for(BusTimetableVO p : times){
			list.add(mapper.map(p, BusTimetableDTO.class));
			
		}
		List<BusTimetableDTO> listLoc = new ArrayList<BusTimetableDTO>(list);
		for(BusTimetableDTO b : listLoc){
			if(b.getTimeCode() == null) continue;
			for(TimeCodeItemDTO i : b.getTimeCode().getItems()){
				//pokud je čaosvý údaj který hledám je v rozmezí from <-> to a má excluding tak v té době nejede
				TimeRange range = new TimeRange(i.getFrom_(), i.getTo_());
				if(range.includes(day) && i.getType().equals("EXCLUDING"))
					list.remove(b);
				else if(!range.includes(day) && i.getType().equals("INCLUDING")){  //pokud je čaosvý údaj který hledám neni v rozmezí from <-> to a má including tak v té době nejede
					list.remove(b);
				}
			}
		}
		return sortListToMap(list);
	}


	private Map<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>> sortListToMap(
			List<BusTimetableDTO> times) {
		Map<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>> map = new LinkedHashMap<Tuple<CityDTO, CityDTO>, List<BusTimetableDTO>>();
		
		if(times.size()==0) return map;
		for(BusTimetableDTO time : times){
			Tuple<CityDTO, CityDTO> key = new Tuple<CityDTO, CityDTO>(time.getDepartureStation().getCity(),time.getArrivalStation().getCity());
				if(map.containsKey(key)){ 
					List<BusTimetableDTO> value = map.get(key);
					if(value.contains(time)) continue;
					value.add(time);
					map.put(key, value);
				}else{
					List<BusTimetableDTO> value = new ArrayList<BusTimetableDTO>();
					value.add(time);
					map.put(key, value);
					//pokusim se hned vytvorit opacny smer aby byly smery tam z5 ve vypisu pohromade
					key = new Tuple<CityDTO, CityDTO>(time.getArrivalStation().getCity(),time.getDepartureStation().getCity());
					if(!map.containsKey(key)){ 
					  value = new ArrayList<BusTimetableDTO>();
					  map.put(key, value);
					}
				}
		}
		return map;
	}
	
}
