package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.annotation.concurrent.Immutable;

import org.dozer.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.ImmutableList;

import cz.sa.shiftPlanner.dao.LineGroupDAO;
import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.LineGroupDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.entities.BusConnectionVO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.utils.LineGroupCitiesSingl;
import cz.sa.shiftPlanner.utils.ServiceFailureException;
import cz.sa.shiftPlanner.ybus.sync.ExceptStrings;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class LineGroupServiceImpl implements LineGroupService {

	private LineGroupDAO lineGroupDAO;
    private Mapper mapper;
    
    @Autowired 
    @Qualifier("lineGroupCities")
    LineGroupCitiesSingl singelton; 
    
	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}


	public void setLineGroupDAO(LineGroupDAO lineGroupDAO) {
		this.lineGroupDAO = lineGroupDAO;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public LineGroupDTO get(Long id) {
		
		return mapper.map(lineGroupDAO.get(id),LineGroupDTO.class);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void create(LineGroupDTO line) {
		
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void update(LineGroupDTO line) {
		
	}

	@Override
	@Transactional
	public void remove(LineGroupDTO line) {
		lineGroupDAO.remove(lineGroupDAO.get(line.getId()));
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public List<LineGroupDTO> findAll() {
		List<LineGroupDTO> list = new ArrayList<LineGroupDTO>();
		for(LineGroupVO vo : lineGroupDAO.findAll())
			list.add(mapper.map(vo, LineGroupDTO.class));
		return list;
	}

	private List<LineGroupDTO> getCroppedList(List<LineGroupDTO> input){
		List<LineGroupDTO> result = input;
		for(LineGroupDTO line : ImmutableList.copyOf(input)){
			for (String except : ExceptStrings.exceptions) {
				if(line.getName().toLowerCase().contains(except.toLowerCase())){
					result.remove(line);	
					break;
				}
			}
		}
		return result;
	}

	@Override
	public List<LineGroupDTO> findByPerson(UserDTO loggedUser) {
		
		//pokud se jedna o jakehokoli jineho uzivatele nez je steward vracim seznam relevantnich linek (neberu v potaz vlaky, letadla, ..)
		if(!loggedUser.getRole().equalsIgnoreCase("steward"))
			return getCroppedList();
		List<LineGroupDTO> lineGroups = new ArrayList<LineGroupDTO>();
		for(LineGroupVO l : singelton.getLinesByCities(loggedUser.getPerson().getCity())){
		  lineGroups.add(mapper.map(l, LineGroupDTO.class));
		}
		return getCroppedList(lineGroups);
	}


  @Override
  public List<LineGroupDTO> getCroppedList() {
    return getCroppedList(findAll());
  }

}
