package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.List;

import org.dozer.Mapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.NotificationDAO;
import cz.sa.shiftPlanner.dto.NotificationDTO;
import cz.sa.shiftPlanner.dto.NotificationLoadDTO;
import cz.sa.shiftPlanner.entities.NotificationVO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.services.NotificationService;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class NotificationServiceImpl implements NotificationService {

	private Mapper mapper;
	private NotificationDAO notificationDAO;
	
	
	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}

	public void setNotificationDAO(NotificationDAO notificationDAO) {
		this.notificationDAO = notificationDAO;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void create(NotificationDTO note) {

		NotificationVO n = mapper.map(note, NotificationVO.class);
		
		notificationDAO.create(n);
		mapper.map(n, note);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void update(NotificationDTO note) {
		
		notificationDAO.update(mapper.map(note, NotificationVO.class));
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void remove(NotificationDTO note) {

		notificationDAO.remove(mapper.map(note, NotificationVO.class));
	}

	@Override
	public List<NotificationDTO> findSelectedBy(NotificationLoadDTO load) {

		List<NotificationDTO> list = new ArrayList<NotificationDTO>();
		for(NotificationVO n : notificationDAO.findSelectedBy(load)){
			list.add(mapper.map(n, NotificationDTO.class));
		}
		return list;
	}

	@Override
	public boolean checkUnreaded() {
		
		return notificationDAO.checkUnreaded();
	}


}
