package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.List;

import org.dozer.Mapper;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.BusTurnsTimetableDAO;
import cz.sa.shiftPlanner.dao.PersonDAO;
import cz.sa.shiftPlanner.dao.RuleDAO;
import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.RuleDTO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.RuleVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.RuleService;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class RuleServiceImpl implements RuleService {

	private RuleDAO ruleDAO;
	private PersonDAO personDAO;
	private BusTurnsTimetableDAO busTurnsTimetableDAO;
	
	private Mapper mapper; 
	
	public Mapper getMapper() {
		return mapper;
	}

	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}

	public void setPersonDAO(PersonDAO personDAO) {
		this.personDAO = personDAO;
	}
	
	public void setBusTurnsTimetableDAO(BusTurnsTimetableDAO busTurnsTimetableDAO) {
		this.busTurnsTimetableDAO = busTurnsTimetableDAO;
	}

	public void setRuleDAO(RuleDAO ruleDAO) {
		this.ruleDAO = ruleDAO;
	}


	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void create(RuleDTO rule) {

		if(rule == null) throw new IllegalArgumentException("rule is null");
		RuleVO r = new RuleVO();
		r = mapper.map(rule, RuleVO.class);
		ruleDAO.create(r);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void update(RuleDTO rule) {

		 throw new IllegalArgumentException("rule is null");
		//ruleDAO.update(r);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void remove(RuleDTO rule) {
		ruleDAO.remove(ruleDAO.get(rule.getId()));
		if(rule.getTurn().getTimetable().getLine().isInterstate()){
		  RuleVO existed = ruleDAO.findByWeekAndTurn(mapper.map(rule.getTurn(), BusTurnsTimetableVO.class), rule.getWeek()+2);
  		if(existed != null){
  		  ruleDAO.remove(existed);
  		}
		}
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public List<RuleDTO> findByWeekAndDay(WeekDay day, Integer week, DateTime validity) {

		List<RuleDTO> list = new ArrayList<RuleDTO>();
		for(RuleVO vo : ruleDAO.findByWeekAndDay(day, week, validity))
			list.add(mapper.map(vo, RuleDTO.class));
		
		return list;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public RuleDTO findByWeekAndTurn(BusTurnsTimetableDTO turn, Integer week) {

		RuleVO rule = ruleDAO.findByWeekAndTurn(mapper.map(turn, BusTurnsTimetableVO.class), week);
		if(rule == null) return null;
		else return mapper.map(rule,RuleDTO.class);
			
	}

	@Override
	public RuleDTO create(BusTurnsTimetableDTO busT, int week, PersonDTO per) {

		RuleDTO rule = new RuleDTO();
		rule.setWeek(week);
		rule.setTurn(busT);
		rule.setPerson(per);
		create(rule);
		//pokud jde o vnitro pak je mesic rozdelen jen na sudy a lichy tyden, proto hodnotu prekopiruju
		//if(busT.getTimetable().getArrivalStation().getCity().getCountry().equalsIgnoreCase("cz") && busT.getTimetable().getDepartureStation().getCity().getCountry().equalsIgnoreCase("cz")){
		if(busT.getTimetable().getLine().isInterstate()){
  		  RuleDTO ruleCZ = new RuleDTO();
  			ruleCZ = rule;
  			ruleCZ.setWeek(week+2);
  			create(ruleCZ);
		}
		RuleVO result = ruleDAO.findByWeekAndTurn(mapper.map(busT, BusTurnsTimetableVO.class), week);
		if(result == null)
		  throw new NullPointerException("Nepovedlo se vytvorit Rule");
		busT.getRules().add(mapper.map(result, RuleDTO.class));
		return mapper.map(result, RuleDTO.class);
	}

	@Override
	public void createAll(List<BusTurnsTimetableDTO> list, Integer week, PersonDTO per) {

		for(BusTurnsTimetableDTO busT : list){
		  RuleVO rule = ruleDAO.findByWeekAndTurn(mapper.map(busT, BusTurnsTimetableVO.class), week);
		  if(rule == null)
		    create(busT, week, per);
		}
		
	}

}
