package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.List;

import org.dozer.Mapper;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.PersonDAO;
import cz.sa.shiftPlanner.dao.UserDAO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.entities.PersonVO;
import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.entities.infrastructure.UserVO;
import cz.sa.shiftPlanner.services.UserService;
import cz.sa.shiftPlanner.utils.ServiceFailureException;

@Service
@Transactional
public class UserServiceImpl implements UserService {

	private UserDAO userDAO;
	private Mapper mapper;
	private PasswordEncoder passwordEncoder;
	private PersonDAO personDAO;

	public void setPersonDAO(PersonDAO personDAO) {
		this.personDAO = personDAO;
	}

	public void setUserDAO(UserDAO userDAO) {
		this.userDAO = userDAO;
	}

	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}


	public void setPasswordEncoder(PasswordEncoder passwordEncoder) {
		this.passwordEncoder = passwordEncoder;
	}

	@Override
	@Transactional(readOnly=true)
	public UserDTO get(String username) {

		UserVO u = userDAO.get(username);
		if(u == null) 
			return null;
		return mapper.map(u, UserDTO.class);
	}

	@Override
	@Transactional(readOnly=true)
	public UserDTO get(Long id) {

		return mapper.map(userDAO.get(id), UserDTO.class);
	}

	@Override
	@Transactional
	public void create(UserDTO user) throws ServiceFailureException {

		PersonVO p = personDAO.findByPersonalNumber(Long.valueOf(user.getUsername()));
		if(p == null)  throw new ServiceFailureException("personal number doest exists");
		UserVO u = mapper.map(user, UserVO.class);
		u.setPerson(p);
		userDAO.create(u);
		mapper.map(u, user);
	}

	@Override
	@Transactional
	public void remove(UserDTO user) {

		UserVO u = mapper.map(user, UserVO.class);
		userDAO.remove(u);
	}

	@Override
	@Transactional
	public void update(UserDTO user) {

		UserVO u = mapper.map(user, UserVO.class);
		userDAO.update(u);
	}

	@Override
	@Transactional
	public void updatePassword(UserDTO user, String password) {

		UserVO u = userDAO.get(user.getId());        
        String hash = getPasswordHash(u.getUsername(), password);
        u.setPasswordHash(hash);
        userDAO.update(u);
	}

	@Override
	@Transactional(readOnly=true)
	public boolean checkPassword(UserDTO user, String password) {

		if(user == null) return false;
		UserVO u = userDAO.get(user.getId());
		
        String hash = getPasswordHash(u.getUsername(), password);
        return hash.equals(u.getPasswordHash());
	}

	@Override
	@Transactional(readOnly=true)
	public List<UserDTO> findAll() {
		// TODO Auto-generated method stub
		return null;
	}
	
	private String getPasswordHash(String username, String password)
    {
        return passwordEncoder.encodePassword(password, username);
    }

	@Override
	public UserDTO findByPerson(PersonVO p) {
		
		UserVO u = userDAO.findByPerson(p);
		if(u == null) return null;
		return mapper.map(u,UserDTO.class);
	}

	@Override
	public List<UserDTO> findSingIn() {
		
		List<UserDTO> result = new ArrayList<UserDTO>();
		for(UserVO u : userDAO.findSignIn()){
			result.add(mapper.map(u, UserDTO.class));
		}
		return result;
	}

}
