package cz.sa.shiftPlanner.servicesImpl;


import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.LineGroupDTO;
import cz.sa.shiftPlanner.dto.NotificationDTO;
import cz.sa.shiftPlanner.dto.RuleDTO;
import cz.sa.shiftPlanner.dto.StationDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.BusConnectionService;
import cz.sa.shiftPlanner.services.NotificationService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.RuleService;
import cz.sa.shiftPlanner.services.WayService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;

@Service
public class WayGenerator {

	@Autowired
	@Qualifier("ruleService")
	RuleService ruleService;
	
	@Autowired
	WayService wayService;
	
	@Autowired
	PersonService personService;
	
	@Autowired
	@Qualifier("busConnectionService")
	BusConnectionService connectionService;
	
	@Autowired
	@Qualifier("notificationService")
	NotificationService notificationService;
	
	Logger log = Logger.getLogger(WayGenerator.class);


	public int generateWay(DateTime from, DateTime to){
		
		log.info("zacatek generovani spoju");
		
		int wayCount = 0;
		DateTime to_ = to;
		DateTime generateTime = from;
		
		List<WeekDay> list = Arrays.asList(WeekDay.values());
		
		int weekIndex = DateFormatter.getWeekMod4(generateTime);
		
		while(generateTime.compareTo(to_) <= 0){
			
			//najde pravidla podle tydne
			List<RuleDTO> rules = ruleService.findByWeekAndDay(list.get(generateTime.getDayOfWeek()-1), weekIndex, generateTime);
			
			for(RuleDTO rule : rules){
				StationDTO departureStation =  rule.getTurn().getTimetable().getDepartureStation();
				StationDTO arrivalStation = rule.getTurn().getTimetable().getArrivalStation();
				LineGroupDTO line = rule.getTurn().getTimetable().getLine();
				DateTime departure = new DateTime(
						generateTime.getYear(), 
						generateTime.getMonthOfYear(), 
						generateTime.getDayOfMonth(), 
						rule.getTurn().getTimetable().getDepartureTime().getHourOfDay(), 
						rule.getTurn().getTimetable().getDepartureTime().getMinuteOfHour(),
						0, 0);
				
				DateTime arrival = new DateTime(
						generateTime.getYear(), 
						generateTime.getMonthOfYear(), 
						generateTime.getDayOfMonth(), 
						rule.getTurn().getTimetable().getArrivalTime().getHourOfDay(), 
						rule.getTurn().getTimetable().getArrivalTime().getMinuteOfHour(),
						0, 0);
//				System.out.println("d: "+departure);
//				System.out.println("a: "+arrival+"\n");
				BusConnectionDTO connection = connectionService.findConnectionByConcreteFigure(departure, arrival, departureStation, arrivalStation, line);
				if(connection != null){
					if(connection.getWays().size()==0){
						WayDTO way = new WayDTO();
						way.setArrivalStation(arrivalStation);
						way.setArrivalTime(arrival);
						way.setDepartureStation(departureStation);
						way.setDepartureTime(departure);
						way.setDelay(0);
						way.setPerson(rule.getPerson());
						way.setConnection(connection);
						way.setIsForeign(!arrivalStation.getCity().getCountry().equalsIgnoreCase("CZ"));
						log.info(connection.getId() + " spoj nalezen a vygenerovan");
						try {
							wayService.create(way);
							wayCount++;
						} catch (ServiceNoticeException e) {
							NotificationDTO note = new NotificationDTO();
							note.setCreated(new DateTime());
							note.setReaded(false);
							note.setPerson(personService.findByPersonalNumber(4L));
							note.setNotifType(NotifType.GENERATOR);
							note.setText(way.getPerson().toString() + ": "+e.getMessage());
							log.info(note.getText());
							notificationService.create(note);
						}
					}
					//TODO otazka jestli toto delat. Prepisovani uz nagenerovanych spoju pro aktualizaci podle noveho planu
/*					if(connection.getWays().size() == 1){
						WayDTO way = connection.getWays().get(0);
						way.setArrivalStation(arrivalStation);
						way.setArrivalTime(arrival);
						way.setDepartureStation(departureStation);
						way.setDepartureTime(departure);
						way.setDelay(0);
						way.setPerson(rule.getPerson());
						way.setConnection(connection);
						way.setIsForeign(!arrivalStation.getCity().getCountry().equalsIgnoreCase("CZ"));
						log.info(connection.getId() + " spoj nalezen a prepsan");
						wayService.update(way);
					}
*/			}
			}
			generateTime = generateTime.plusDays(1);
			weekIndex = DateFormatter.getWeekMod4(generateTime);
		}
		log.info("konec generovani spoju");
		return wayCount;
	}


	public void setRuleService(RuleService ruleService) {
		this.ruleService = ruleService;
	}


	public void setConnectionService(BusConnectionService connectionService) {
		this.connectionService = connectionService;
	}


	public void setWayService(WayService wayService) {
		this.wayService = wayService;
	}
	
	
}
