package cz.sa.shiftPlanner.ybus.sync;

import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.MessageContext;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.apache.log4j.Logger;
import org.dozer.Mapper;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.BusConnectionDAO;
import cz.sa.shiftPlanner.dao.BusTimetableDAO;
import cz.sa.shiftPlanner.dao.BusTurnsTimetableDAO;
import cz.sa.shiftPlanner.dao.CityDAO;
import cz.sa.shiftPlanner.dao.LineGroupDAO;
import cz.sa.shiftPlanner.dao.RuleDAO;
import cz.sa.shiftPlanner.dao.TimeCodeDAO;
import cz.sa.shiftPlanner.dto.NotificationDTO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.services.NotificationService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.ybus.api.ObjectFactory;
import cz.sa.ybus.api.YServiceShiftPlanner;
import cz.sa.ybus.api.YServiceShiftPlanner_Service;

/**
 * @author ToSk
 */

@Service
public abstract class SyncAction implements SyncInterface{

	private static final Logger log = Logger.getLogger(SyncAction.class);
	
	@Autowired
	  ObjectFactory factory = new ObjectFactory();
	@Autowired
  @Qualifier(value = "lineGroupDAO")
	  LineGroupDAO lineGroupDAO;
	@Autowired
  @Qualifier("notificationService")
    NotificationService notificationService;
  @Autowired
  @Qualifier(value = "personService")
    PersonService personService;
  @Autowired
  @Qualifier("busConnectionDAO")
    BusConnectionDAO busConnectionDAO;
  @Autowired
  @Qualifier("busTimetableDAO")
    BusTimetableDAO busTimetableDAO;
  @Autowired
  @Qualifier(value = "cityDAO")
    CityDAO cityDAO;
  @Autowired
  @Qualifier(value = "timeCodeDAO")
    TimeCodeDAO timeCodeDAO;
  @Autowired
  @Qualifier("busTurnsTimetableDAO")
    BusTurnsTimetableDAO busTurnsTimetableDAO;
  @Autowired
  @Qualifier(value = "mapper")
    Mapper mapper;
  @Autowired
  @Qualifier("ruleDAO")
    RuleDAO ruleDAO;
  
	private YbusSyncPlan plan;
	private SyncInterface parent;
	private Set<SyncInterface> processedDependant = new HashSet<SyncInterface>();
	private Set<SyncInterface> dependent = new HashSet<SyncInterface>();
	//private Set<SyncAction> consequences = new HashSet<SyncAction>();
	
	YServiceShiftPlanner_Service plannerService;

	private YServiceShiftPlanner service;

	{
		URL url = null;
		try {
			url = new URL("");
		} catch (MalformedURLException e) {
			log.warn("Failed to create URL for the wsdl Location: 'YServiceShiftPlanner.wsdl', retrying as a local file");
			log.warn(e.getMessage());
		}
		plannerService = new YServiceShiftPlanner_Service(url, new QName("cz.sa.ybus.search", "YServiceShiftPlanner"));
		YServiceShiftPlanner port = plannerService.getPort(new QName("cz.sa.ybus.search", "YServiceShiftPlanner"), YServiceShiftPlanner.class);
		Client client = ClientProxy.getClient(port);
		HTTPConduit httpConduit = (HTTPConduit) client.getConduit();

		HTTPClientPolicy httpClientPolicy = new HTTPClientPolicy();
		httpClientPolicy.setConnectionTimeout(36000);
		httpClientPolicy.setReceiveTimeout(32000);
		httpClientPolicy.setAllowChunking(false);
		addPortProperties((BindingProvider) port);
		httpConduit.setClient(httpClientPolicy);
		service = port;
	}
	
	
	public YServiceShiftPlanner getService() {
		return service;
	}
	
	public YbusSyncPlan getPlan() {
		return plan;
	}
	public void setPlan(YbusSyncPlan plan) {
		this.plan = plan;
	}
	public Set<SyncInterface> getDependent() {
		return dependent;
	}
	
	
	@Override
	public void finish(SyncAction action) {
		processedDependant.remove(action);
		if(processedDependant.size() == 0){
//		  try{
		    run();
		    parent.finish(this);
//		  }catch(Exception ex){
//		    log.error("Method run in class "+this.getClass() + " throw exception:", ex);
//				return;
//		  }
		}
	}
	
	@Override
	public final void start(SyncInterface syncInterface){
		this.parent = syncInterface;
		if(dependent.size() == 0){
			finish(null);
		}else{
			processedDependant.addAll(dependent);
			for (SyncInterface action : dependent) {
				action.start(this);
			}
		}
	}
	
	private void addPortProperties(BindingProvider provider) {
    final Map<String, Object> requestContext = provider.getRequestContext();
    requestContext.put(BindingProvider.USERNAME_PROPERTY, "");
    requestContext.put(BindingProvider.PASSWORD_PROPERTY, "");
  }
	
	protected void createNotification(String text) {
    NotificationDTO note = new NotificationDTO();
    note.setCreated(new DateTime());
    note.setReaded(false);
    note.setPerson(personService.findByPersonalNumber(4L));
    note.setNotifType(NotifType.SYS_CHYBY);
    note.setText(text);
    notificationService.create(note);
  }
	
	public abstract void run();
}


