package cz.sa.shiftPlanner.ybus.sync;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dto.NotificationDTO;
import cz.sa.shiftPlanner.entities.BusConnectionVO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.utils.ServiceFailureException;
import cz.sa.ybus.api.BusConnection;
import cz.sa.ybus.api.BusConnectionsFilter;
import cz.sa.ybus.api.BusConnectionsFilter.Connections;
import cz.sa.ybus.api.Changed;
import cz.sa.ybus.api.ConnectionsRequest;
import cz.sa.ybus.api.ConnectionsResponse;
import cz.sa.ybus.api.Created;
import cz.sa.ybus.api.DateTimeFilter;
import cz.sa.ybus.api.ResultCodes;

/**
 * @author ToSk
 */

@Service
//@Transactional(propagation = Propagation.REQUIRES_NEW)
public class SyncBusConnections extends SyncAction {

	private static final Logger log = Logger.getLogger(SyncBusConnections.class);
	
	private @Autowired @Qualifier("syncTimetables") SyncTimetable timetable;
	private Set<Long> timIds = new HashSet<Long>();
	private Set<Long> createdIds = new HashSet<Long>();
	
	/**
	 * 
	 * @param after je v ybusu zpracovan takto:  (dateCreated > :after)
	 * @param before je v ybusu zpracovan takto: (dateCreated < :before)
	 * @param auditsOnly
	 * @return List<{@link BusConnection}>
	 */
	private List<BusConnection> getBusConnectionsAfter(DateTimeFilter timeFilter) {
	  log.info("get busConnections start");
		ConnectionsRequest request = factory.createConnectionsRequest();
		BusConnectionsFilter filter = factory.createBusConnectionsFilter();
		Connections value = new Connections();
		value.getConnectionId().addAll(createdIds);
		filter.setConnections(value);
		
		ConnectionsResponse response;
		List<BusConnection> list = new ArrayList<BusConnection>();

		do {
		  if(timeFilter instanceof Created)
		    filter.setCreated((Created)timeFilter);
		  if(timeFilter instanceof Changed)
		    filter.setChanged((Changed)timeFilter);
			request.setBusConnectionsFilter(filter);
			response = getService().getTimesheetData(request);
			list.addAll(response.getBusConnections().getBusConnection());
			if (list.size() > 5)
				timeFilter.setAfter(list.get(list.size() - 2).getFilterDate());
		} while (response.getResult().getCode().compareTo(ResultCodes.EXCEEDED) == 0);

		log.info("get busConnections end");
		return list;
	}
	
	private void downloadBusConnectionsCreated(){
	  log.info("download busConnections start");
	  Created created = factory.createCreated();
    created.setAfter(new DateTime().minusMonths(2));
    created.setBefore(new DateTime());
		for(BusConnection c : getBusConnectionsAfter(created)){
			if(c != null){
				BusConnectionVO vo = busConnectionDAO.get(c.getId());
				//nemam spoj ulozeny v dtb => vytvorim novy
				if(vo == null){
					try {
						checkUnwantedLG(c);
					} catch (ServiceFailureException e) {
					  log.info(e.getMessage());
						continue;
					}
					vo = new BusConnectionVO();
					if(c.getStandard() == null)
						vo.setStandard("YELLOW");
					else
						vo.setStandard(c.getStandard().name());
					vo.setArrivalTime(c.getArrivaltime());
					vo.setDepartureTime(c.getDeparturetime());
					vo.setId(c.getId());
					BusTimetableVO tim = busTimetableDAO.get(c.getConnectionId());
					if(tim == null){
					  timIds.add(c.getConnectionId());
						continue;
					}
					vo.setTimetable(tim);
					
					if(c.getSupportforId() != null){
						BusConnectionVO conn = busConnectionDAO.get(c.getSupportforId());
						if(conn == null){
							createNotification("Puvodni spoj s id="+c.getSupportforId()+" pro posilu neni v databazi.");
							log.error("Puvodni spoj s id="+c.getSupportforId()+" pro posilu neni v databazi.");
							continue;
						}
						vo.setSupportFor(conn);
					}
					busConnectionDAO.save(vo);
				}else{
				  //spoj mam v databazi => netyka se to metody getCreated
				}
			}
		}
		log.info("download connections ends");
	}

	private void downloadBusConnectionsChanged(){
	  log.info("download changed busConnections start");
	  Created changed = factory.createCreated();
	  changed.setAfter(new DateTime().minusMonths(2));
	  changed.setBefore(new DateTime());
    for(BusConnection c : getBusConnectionsAfter(changed)){
      if(c != null){
        BusConnectionVO vo = busConnectionDAO.get(c.getId());
        //spoj mam v dtb => update
        if(vo != null ){
      	  vo.setArrivalTime(c.getArrivaltime());
          vo.setDepartureTime(c.getDeparturetime());
          vo.setStandard(c.getStandard().name());
          if(c.getSupportforId() != null){
            BusConnectionVO conn = busConnectionDAO.get(c.getSupportforId());
            if(conn == null){
              createNotification("Puvodni spoj s id="+c.getSupportforId()+" pro posilu neni v databazi.");
              log.error("Puvodni spoj s id="+c.getSupportforId()+" pro posilu neni v databazi.");
              continue;
            }
            vo.setSupportFor(conn);
          }
         busConnectionDAO.update(vo);
        }
      }
    }
    log.info("download changed busConnections end");
	}
	
	private void checkUnwantedLG(BusConnection c) throws ServiceFailureException {
		LineGroupVO line = lineGroupDAO.get(c.getLineGroup());
		if(line == null){
			createNotification("Skupina linek s id="+c.getLineGroup()+" neexistuje v databazi shiftplanner.");
			log.error("Skupina linek s id="+c.getLineGroup()+" neexistuje v databazi shiftplanner.");
			throw new ServiceFailureException("LineGroup is null");
		}
		for (String except : ExceptStrings.exceptions) {
			if(line.getName().toLowerCase().contains(except.toLowerCase()))
				throw new ServiceFailureException("Line group is unwanted");	
		}
	}
	
	@Override
	public void run() {
		log.info("run connections");
		downloadBusConnectionsChanged();
//		downloadBusConnectionsCreated();
//		if(timIds.size() != 0){
//		  log.info("missed timetables field size: "+timIds.size());
//  		createdIds = timetable.downloadByIds(timIds);
//  		log.info(createdIds.size()+" missed timetables was created");
//  		downloadBusConnectionsCreated();
//		}
//		timIds = new HashSet<Long>();
//	  createdIds = new HashSet<Long>();
	}


}


