package cz.sa.shiftPlanner.web;

import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.LineGroupDTO;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.wizard.RulesWizard;
import cz.sa.shiftPlanner.web.wizard.WizardPage;

public class LineGroupViewPage extends MenuPage {

	@SpringBean(name = "lineGroupService")
    LineGroupService lineService;

	public LineGroupViewPage(PageParameters parameters) {
		super(parameters);

		showAll();
	}

	private void showAll() {

		add(new Label("message", "Seznam skupin linek"));

		
		add(new ShowForm("lineForm"));
	}
	
	private class ShowForm extends Form<Void>{
		
		public ShowForm(String id){
			super(id);
			
			Component header = new AbstractItem("header");
			add(header);
			RepeatingView repeatingView = new RepeatingView("repeating");
			add(repeatingView);

			IModel<List<LineGroupDTO>> model = new LoadableDetachableModel<List<LineGroupDTO>>() {
				@Override
				protected  List<LineGroupDTO> load(){
					return Lists.newArrayList(lineService.findByPerson(((PlannerSession)PlannerSession.get()).getLoggedUser()));
				}
			};
			
			int i=0;
			for (final LineGroupDTO lin : model.getObject()) {
				AbstractItem item = new AbstractItem(repeatingView.newChildId());

				item.add(new Label("name", String.valueOf(lin.getName())));
				
				
				Link button = new Link("timetable", new Model("")){
					@Override
					public void onClick() {
						setResponsePage(TimetableViewPage.class, fillParameters(0, lin));
					}
				};
				item.add(button);
				
				Link button1 = new Link("turns", new Model("")){
					@Override
					public void onClick() {
						setResponsePage(WizardPage.class, fillParameters(1, lin));
					}
				};
				MetaDataRoleAuthorizationStrategy.authorize(button1, RENDER, PlannerRoles.COORDINATOR);
				item.add(button1);
				
				Link button2 = new Link("rules", new Model("")){
					@Override
					public void onClick() {
						setResponsePage(WizardPage.class, fillParameters(2, lin));
					}
				};
				MetaDataRoleAuthorizationStrategy.authorize(button2, RENDER, PlannerRoles.COORDINATOR);
				item.add(button2);
				
				Link button3 = new Link("ways", new Model("")){
					@Override
					public void onClick() {
						setResponsePage(WizardPage.class, fillParameters(3, lin));
					}
				};
				item.add(button3);

				if(i%2==0) item.add(new AttributeModifier("class", "even"));
				else item.add(new AttributeModifier("class", "odd"));
				
				i++;
				
				
				repeatingView.add(item);
			}
			
		}
		
	}
	
	private PageParameters fillParameters(int number, final LineGroupDTO lin){
		PageParameters pageParameters = new PageParameters();
		pageParameters.add("id", String.valueOf(lin.getId()));
		pageParameters.add("page", number);
		return pageParameters;
	}
}
