package cz.sa.shiftPlanner.web;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Page;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.springframework.beans.factory.annotation.Configurable;

import cz.sa.shiftPlanner.services.NotificationService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.mailer.MailerPage;
import cz.sa.shiftPlanner.web.notification.NotificationModalWindow;
import cz.sa.shiftPlanner.web.vacation.VacationViewPage;

@Configurable(preConstruction = true)
public class MenuPage extends HomePage {

  @SpringBean(name = "personService")
  PersonService personService;

  @SpringBean(name = "notificationService")
  NotificationService notificationService;

  private PageParameters superPageParam;

  public MenuPage(PageParameters parameters) {
    super(parameters);

    this.superPageParam = parameters;
    showAll();
  }

  private void showAll() {
    final PageParameters param = new PageParameters();
    PropertyModel<String> userNumber = new PropertyModel<String>(this, "session.userNumber");
    param.add("perId", String.valueOf(personService.findByPersonalNumber(Long.valueOf(userNumber.getObject())).getId()));

    add(new BookmarkablePageLink("lines", LineGroupViewPage.class));
    add(new BookmarkablePageLink("support", SupportViewPage.class).setVisible(false));
    BookmarkablePageLink personCart = new BookmarkablePageLink("personCart", PersonDetailPage.class, param);
    personCart.setVisible(PlannerSession.get().getRoles().contains("steward"));
    add(personCart);
    BookmarkablePageLink personPage = new BookmarkablePageLink("person", PersonViewPage.class);
    MetaDataRoleAuthorizationStrategy.authorize(personPage, RENDER, new String(PlannerRoles.COORDINATOR + ", "+ PlannerRoles.HELPLINKA));
    add(personPage);
    
    BookmarkablePageLink vacationPage = new BookmarkablePageLink("vacation", VacationViewPage.class);
    MetaDataRoleAuthorizationStrategy.authorize(vacationPage, RENDER, new String(PlannerRoles.COORDINATOR + ", "+ PlannerRoles.HELPLINKA));
    add(vacationPage);
    
    BookmarkablePageLink mailer = new BookmarkablePageLink("mailer", MailerPage.class);
    MetaDataRoleAuthorizationStrategy.authorize(mailer, RENDER, new String(PlannerRoles.COORDINATOR + ", "+ PlannerRoles.HELPLINKA));
    add(mailer);

    BookmarkablePageLink servicePage = new BookmarkablePageLink("servicePage", ServicePage.class);
    add(servicePage);
    MetaDataRoleAuthorizationStrategy.authorize(servicePage, RENDER, PlannerRoles.COORDINATOR);

    BookmarkablePageLink userPage = new BookmarkablePageLink("userPage", UserPage.class);
    userPage.setVisible(false);
    add(userPage);
    MetaDataRoleAuthorizationStrategy.authorize(userPage, RENDER, PlannerRoles.SUPERVISOR);

    Link editPass = new Link("changePassword") {
      @Override
      public void onClick() {
        setResponsePage(new UserPage(new PageParameters(), true));
      }
    };
    add(editPass);
    
    BookmarkablePageLink test = new BookmarkablePageLink("test", SupportViewPage.class);
    test.setVisible(false);
    add(test);

    // The ModalWindow, showing some choices for the user to select.
    final ModalWindow selectModalWindow = new ModalWindow("modalwindow");
    selectModalWindow.setCookieName("modal-notif");
    selectModalWindow.setInitialWidth(1024);
    selectModalWindow.setInitialHeight(768);
    selectModalWindow.setTitle("Upozoreni");
    selectModalWindow.setVersioned(true);
    add(selectModalWindow);

    selectModalWindow.setPageCreator(new ModalWindow.PageCreator() {

      @Override
      public Page createPage() {
        return new NotificationModalWindow(MenuPage.this.getPageReference(), selectModalWindow, superPageParam);
      }
    });

    AjaxLink notification = new AjaxLink("linkToModalWindow") {
      public void onClick(AjaxRequestTarget target) {
        selectModalWindow.show(target);
      }
    };
    Label text = new Label("modalText", "Oznameni");
    //        if(notificationService.checkUnreaded())
    //        	text.add(new AttributeModifier("id", "notification"));

    notification.add(text);
    add(notification);
    MetaDataRoleAuthorizationStrategy.authorize(notification, RENDER, new String(PlannerRoles.COORDINATOR + ", "+ PlannerRoles.HELPLINKA));
  }
}
