package cz.sa.shiftPlanner.web;

import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.datetime.StyleDateConverter;
import org.apache.wicket.datetime.markup.html.form.DateTextField;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.services.BusConnectionService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.servicesImpl.WayGenerator;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.ybus.sync.YbusSyncPlanInterf;

/**
 * @author ToSk
 */

public class ServicePage extends MenuPage {

	@SpringBean(name = "personService")
	PersonService personService;
	
	@SpringBean(name = "generator")
	WayGenerator generator;
	
	@SpringBean(name = "busConnectionService")
	BusConnectionService busConnectionService;
	
	@SpringBean(name = "ybusSyncPlan")
	YbusSyncPlanInterf ybusSyncPlan;
	
	private DateTime actual = new DateTime();
	private IModel<Date> dateModelFrom = new Model<Date>(new Date());
	private IModel<Date> dateModelTo = new Model<Date>( new DateTime(actual.getYear(), actual.plusMonths(1).getMonthOfYear(), 1, 0, 0, 0, 0).toDate());
	
	private static final Logger log = Logger.getLogger(ServicePage.class);
	
	public ServicePage (PageParameters parameters) {
		super(parameters);

		downloadStew();
		ybusSync();
		genWays();
		checkConditions();
	}

	

	private void ybusSync() {
		Link ybusSync = new Link("ybusSync"){
			@Override
			public void onClick() {

				log.info("Stahovani zacalo");
				ybusSyncPlan.runPlan();
				getSession().info("Stahovani ukonceno");
				setResponsePage(ServicePage.class);
			}
		};
		add(ybusSync);
		MetaDataRoleAuthorizationStrategy.authorize(ybusSync, RENDER, PlannerRoles.COORDINATOR);
	}



	private void checkConditions() {
		add(new Link("checkMorning"){
			@Override
			public void onClick() {

				busConnectionService.checkMorning();
				getSession().info("Kontrola ukoncena");
				setResponsePage(ServicePage.class);
			}
		});
		
		add(new Link("checkAfternoon"){
			@Override
			public void onClick() {

				busConnectionService.checkAfternoon();
				getSession().info("Kontrola ukoncena");
				setResponsePage(ServicePage.class);
			}
		});
	}

	private void genWays() {
		Form genForm = new Form("genForm");
		add(genForm);
		
		DateTextField dateTextField = new DateTextField("calendar", dateModelFrom, new StyleDateConverter(false));
		genForm.add(dateTextField);
		
		DatePicker datePicker = new DatePicker()
		{
			@Override
			protected String getAdditionalJavaScript()
			{
				return "${calendar}.cfg.setProperty(\"navigator\",true,false); ${calendar}.render();";
			}
		};
		datePicker.setShowOnFieldClick(true);
		datePicker.setAutoHide(true);
		dateTextField.add(datePicker);

		
		DateTextField dateTextField2 = new DateTextField("calendar2", dateModelTo, new StyleDateConverter(false));
		genForm.add(dateTextField2);
		
		DatePicker datePicker2 = new DatePicker()
		{
			@Override
			protected String getAdditionalJavaScript()
			{
				return "${calendar}.cfg.setProperty(\"navigator\",true,false); ${calendar}.render();";
			}
		};
		datePicker2.setShowOnFieldClick(true);
		datePicker2.setAutoHide(true);
		dateTextField2.add(datePicker2);
		
		final FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        
		genForm.add(new Button("generate"){
			@Override
			public void onSubmit() {

				getSession().info("bylo vygenerovano "+generator.generateWay(new DateTime(dateModelFrom.getObject()), new DateTime(dateModelTo.getObject()))+" spoju.");
				getSession().info("Generovani spoju zkoncilo");
				setResponsePage(ServicePage.class);
			}
		});
	}

	private void downloadStew() {
		Link updateP = new Link("updateP"){
			@Override
			public void onClick() {

				log.info("Stahovani zacalo");
				personService.downloadStewardsFromHelios();
				getSession().info("Stahovani ukonceno");
				setResponsePage(ServicePage.class);
			}
		};
		add(updateP);
		MetaDataRoleAuthorizationStrategy.authorize(updateP, RENDER, PlannerRoles.SUPERVISOR);
	}
	
}


