package cz.sa.shiftPlanner.web;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.cxf.common.util.SortedArraySet;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.yui.calendar.TimeField;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.web.MO.RangeMO;
import cz.sa.shiftPlanner.web.utils.RangeMOComparator;

/**
 * @author ToSk
 */

public class SupportViewPage extends MenuPage{

  private SortedSet<RangeMO> dates = new TreeSet<RangeMO>(new RangeMOComparator());
  
	public SupportViewPage(PageParameters parameters) {
		super(parameters);

		renderDatesList();
		
		renderCalendar();
	}

  private void renderCalendar() {
    WebMarkupContainer calendar = new WebMarkupContainer("fromDate"){
		  @Override
		  protected void onConfigure() {
		    super.onConfigure();
		    RequestCycle cycle = RequestCycle.get();
        WebRequest webRequest = (WebRequest) cycle.getRequest();
        List<StringValue> params = webRequest.getQueryParameters().getParameterValues("date");
        if(params != null && params.size() > 0){
          String[] strings = params.get(params.size()-1).toString().split("\\."); 
          if(strings.length == 3){
            DateTime dateFrom = new DateTime(Integer.parseInt(strings[2]), Integer.parseInt(strings[1]), Integer.parseInt(strings[0]), 0, 0);
            DateTime dateTo = new DateTime(Integer.parseInt(strings[2]), Integer.parseInt(strings[1]), Integer.parseInt(strings[0]), 23, 59);
            dates.add(new RangeMO(new Model<Date>(dateFrom.toDate()), new Model<Date>(dateTo.toDate())));
            renderDatesList();
          }
        }
		  }
		};
    add(calendar);
  }

  private void renderDatesList() {
    final Form datesContainer = new Form("container");
    datesContainer.setOutputMarkupId(true);
    
    ListDataProvider<RangeMO> dp = new ListDataProvider<RangeMO>(Lists.newArrayList(dates));
    final DataView<RangeMO> dataView = new DataView<RangeMO>("dates", dp){
      private static final long serialVersionUID = 1L;
      
      @Override
      protected void populateItem(final Item<RangeMO> item)
      {
        item.add(new Label("date", item.getModelObject().getFrom()));
        TimeField timeFrom = new TimeField("timeFrom", item.getModel().getObject().getFrom());
        timeFrom.add(new AjaxFormComponentUpdatingBehavior("onchange") {
          @Override
          protected void onUpdate(AjaxRequestTarget target) {
          }
        });
        TimeField timeTo = new TimeField("timeTo", item.getModelObject().getTo());
        timeTo.setEnabled(false);
        timeFrom.setEnabled(false);
        item.add(timeFrom, timeTo);

        Link delete = new Link("crossLink") {
          @Override
          public void onClick() {
            dates.remove(item.getModelObject());
            renderDatesList();
          }
        };
        item.add(delete);
      }
     };
     datesContainer.add(dataView);
     addOrReplace(datesContainer);
  }
	
}


