package cz.sa.shiftPlanner.web;


import org.apache.log4j.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.form.validation.EqualPasswordInputValidator;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;

import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.services.UserService;
import cz.sa.shiftPlanner.utils.ServiceFailureException;

/**
 * @author ToSk
 */

public class UserPage extends MenuPage {

	@SpringBean(name = "userService")
    private UserService service;
	
	private static final Logger LOG = Logger.getLogger(UserPage.class);

	public UserPage(PageParameters parameters) {
        super(parameters);

        add(new Label("header","Novy uzivatel"));
        
        final RegistrationForm regForm = new RegistrationForm("regForm");
        add(regForm);
    }
    public UserPage(PageParameters parameters, Boolean b) {
        super(parameters);

        add(new Label("header","Zmena hesla"));
        PropertyModel<String> userNumber = new PropertyModel<String>(this, "session.userNumber");
        UserDTO user = service.get(userNumber.getObject());
        final RegistrationForm regForm = new RegistrationForm("regForm",user);
        add(regForm);
    }

    private class RegistrationForm extends Form<Void> {

        
        private UserDTO user = new UserDTO();
        private FormComponent<String> password;
        private CheckBox admin;
        private TextField<String> userName;
        
        public RegistrationForm(String id) {
            super(id);
            setModel(new CompoundPropertyModel(this));
            userName = new RequiredTextField("userName", new PropertyModel<String>(user, "username"));
            userName.setLabel(new ResourceModel("username.label"));
            password = new PasswordTextField("password", Model.of(""));
            password.setLabel(new ResourceModel("password.label"));
            TextField<String> cpassword = new PasswordTextField("cpassword", Model.of(""));
            cpassword.setLabel(new ResourceModel("controlpassword.label"));
            admin = new CheckBox("admin", Model.of(Boolean.FALSE));
//            admin.setLabel(new ResourceModel("admin.label"));
            
            add(userName);
            add(password);
            add(cpassword);
            add(new EqualPasswordInputValidator(password, cpassword));
            add(admin);
            
            final FeedbackPanel feedback = new FeedbackPanel("feedback");
            feedback.setOutputMarkupId(true);
            add(feedback);
        }
        
        public RegistrationForm(String id, UserDTO user) {
            super(id);
            this.user = user;
            setModel(new CompoundPropertyModel(this));
            userName = new TextField("userName", new PropertyModel<String>(user, "username"));
            userName.setLabel(new ResourceModel("username.label"));
            userName.setEnabled(false);
            password = new PasswordTextField("password", Model.of(""));
            password.setLabel(new ResourceModel("password.label"));
            password.setRequired(false);
            FormComponent<String> cpassword = new PasswordTextField("cpassword", Model.of(""));
            cpassword.setLabel(new ResourceModel("controlpassword.label"));
            cpassword.setRequired(false);
            if(user.getRole().equals("coordinator")) 
                admin = new CheckBox("admin", Model.of(Boolean.TRUE));
            else
                admin = new CheckBox("admin", Model.of(Boolean.FALSE));
//            admin.setLabel(new ResourceModel("admin.label")).setVisible(false);
            admin.setVisible(false);
            
            add(userName);
            add(password);
            add(cpassword);
            add(new EqualPasswordInputValidator(password, cpassword));
            add(admin);
            
            final FeedbackPanel feedback = new FeedbackPanel("feedback");
            feedback.setOutputMarkupId(true);
            add(feedback);
        }

        @Override
        protected void onSubmit() {
            if(user.getId()==null){
                if(admin.getConvertedInput()){
                    user.setRole("coordinator");
                }else{
                	user.setRole("steward");
                }
                try {
					service.create(user);
				} catch (ServiceFailureException e) {
					// TODO Auto-generated catch block
				}
                service.updatePassword(user,password.getValue());
                setResponsePage(new MenuPage(new PageParameters()));
            }else{
            	/*if(admin.getConvertedInput()){
                    user.setRole("coordinator");
                }else{
                	user.setRole("steward");
                }*/
                service.update(user);
                //pokud se heslo nerovna NULL tak updatuj i heslo
                if(password.getValue()!=null){
                    service.updatePassword(user,password.getValue());
                }
                setResponsePage(new MenuPage(new PageParameters()));
            }
        }
        
    }
 }
    
	
