package cz.sa.shiftPlanner.web.auth;

import java.util.Locale;

import org.apache.log4j.Logger;
import org.apache.wicket.Session;
import org.apache.wicket.authroles.authentication.AuthenticatedWebSession;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;
import org.apache.wicket.injection.Injector;
import org.apache.wicket.request.Request;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.services.UserService;

/**
 * 
 * @author ToSk
 */

public class PlannerSession extends AuthenticatedWebSession {

  private final Logger logger = Logger.getLogger(PlannerSession.class);

  @SpringBean(name = "authenticationManager")
  private AuthenticationManager authenticationManager;

  @SpringBean(name = "userService")
  private static UserService userService;

  private String user;
  private static String userNumber;
  private DateTime selectedValidity = new DateTime();
  private static UserDTO loggedUser;
  private int selectedYear = new DateTime().getYear();

  public PlannerSession(Request request) {
    super(request);
    setLocale(new Locale("cs", "CZ"));
    injectDependencies();
    ensureDependenciesNotNull();
  }

  private void ensureDependenciesNotNull() {
    if (authenticationManager == null) {
      throw new IllegalStateException("AdminSession requires an authenticationManager.");
    }
  }

  private void injectDependencies() {
    Injector.get().inject(this);
  }

  @Override
  public void onInvalidate() {
    super.onInvalidate();
    UserDTO u = getLoggedUser();
    u.setSign(false);
    userService.update(u);
    loggedUser = null;
    selectedYear = new DateTime().getYear();
  }

  @Override
  public void invalidate() {
    super.invalidate();
    UserDTO u = getLoggedUser();
    u.setSign(false);
    userService.update(u);
    loggedUser = null;
    selectedYear = new DateTime().getYear();
  }

  @Override
  public void signOut() {
    super.signOut();
    UserDTO u = getLoggedUser();
    u.setSign(false);
    userService.update(u);
    loggedUser = null;
    selectedYear = new DateTime().getYear();
  }

  @Override
  public boolean authenticate(String username, String password) {
    boolean authenticated = false;
    try {
      Authentication authentication = authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(username,
          password));
      SecurityContextHolder.getContext().setAuthentication(authentication);
      authenticated = authentication.isAuthenticated();
      UserDTO u = userService.get(username);
      user = u.getPerson().getName() + " " + u.getPerson().getSurname();
      userNumber = username;
      selectedValidity = new DateTime();
      u.setSign(true);
      userService.update(u);
      logger.info("User " + user + " logged");
    }
    catch (AuthenticationException e) {
      logger.warn("User " + user + " failed to login. Reason: " + e.getMessage(), e);
      authenticated = false;
    }
    return authenticated;
  }

  @Override
  public Roles getRoles() {
    Roles roles = new Roles();
    getRolesIfSignedIn(roles);
    return roles;
  }

  private void getRolesIfSignedIn(Roles roles) {
    if (isSignedIn()) {
      roles.add(getLoggedUser().getRole());
    }
  }

  public static DateTime getSelectedValidity() {
    return get().selectedValidity;
  }

  public static UserDTO getLoggedUser() {
    if(loggedUser == null)
      loggedUser = userService.get(userNumber);
    return get().loggedUser;
  }
  
  public static PlannerSession get(){
    return (PlannerSession) Session.get();
  }

  public int getSelectedYear() {
    return selectedYear;
  }

  public void setSelectedYear(int selectedYear) {
    this.selectedYear = selectedYear;
  }
  
  
}
