package cz.sa.shiftPlanner.web.mailer;

import java.io.Serializable;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.springframework.beans.factory.annotation.Configurable;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.pcomponent.PFeedbackPanel;
import cz.sa.shiftPlanner.services.CityService;
import cz.sa.shiftPlanner.services.MailService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.web.MenuPage;
import cz.sa.shiftPlanner.web.auth.PlannerSession;

/**
 *
 * @author    ToSk
 * @version   1.0
 * @created   1. 10. 2013
 */

@Configurable(preConstruction=true)
public class MailerPage extends MenuPage {

  @SpringBean(name = "mailerService")
  private MailService mailService;
  @SpringBean(name = "cityService")
  private CityService cityService;
  @SpringBean(name = "personUtilsService")
  private PersonUtilsService personUtilsService;
  
  private PersonDTO loggedPerson = ((PlannerSession)PlannerSession.get()).getLoggedUser().getPerson();
  private MailMO mailMO = new MailMO();
  
  public MailerPage(PageParameters parameters) {
    super(parameters);

    add(new PFeedbackPanel("feedbackPanel").setOutputMarkupId(true));
    
    add(mailSendForm("mailForm"));
  }

  private Component mailSendForm(String id) {

    Form mailForm = new Form(id);
    
    Button submitButton = new Button("buttonWithYesNO") {
     @Override
     public void onSubmit() {
         if(mailMO.getCity() != null){
           mailService.sendByCity(mailMO.getCity(), mailMO.getMailText(), loggedPerson);
           info("Mail byl uspesne odeslan stevadrum, kteri maji vyjezdove mesto "+ mailMO.getCity().getCityName());
           mailMO = new MailMO();
           return;
         }
         if(mailMO.getAllStews() != null && mailMO.getAllStews()){
           mailService.sendToAllStew(mailMO.getMailText(), loggedPerson);
           info("Mail byl uspesne odeslan vsem stevadrum.");
           mailMO = new MailMO();
           return;
         }
         if(mailMO.getJobTime() != null){
           mailService.sendByJobTime(mailMO.getJobTime(), mailMO.getMailText(), loggedPerson);
           info("Mail byl uspesne odeslan stevadrum, kteri maji pracovni uvazek "+ mailMO.getJobTime());
           mailMO = new MailMO();
           return;
         }
         error("Musi byt zvolena prave jedna moznost na odeslani");
      }
    };
    submitButton.add(new AttributeModifier("onclick", "if(!confirm('Opravdu chcete odeslat maily?')) return false;"));
    mailForm.add(submitButton);
    
    mailForm.add(new CheckBox("allStewards", new PropertyModel<Boolean>(mailMO, "allStews")));
    mailForm.add(new DropDownChoice<CityDTO>("stewardsByCity", new PropertyModel<CityDTO>(mailMO, "city"), cityService.findAllCZ()));
    mailForm.add(new DropDownChoice<String>("byTimeJob", new PropertyModel<String>(mailMO, "jobTime"), Lists.newArrayList("HPP", "DPC")));
    mailForm.add(new TextArea<String>("mailText", new PropertyModel<String>(mailMO, "mailText")).setRequired(true));
    
    return mailForm;
  }

  private class MailMO implements Serializable{
    
    private CityDTO city;
    private String jobTime;
    private Boolean allStews;
    private String mailText;
    
    public MailMO() {
      city = null;
      allStews = null;
      mailText = null;
      jobTime = null;
    }
    
    
    public String getJobTime() {
      return jobTime;
    }


    public void setJobTime(String jobTime) {
      this.jobTime = jobTime;
    }


    public CityDTO getCity() {
      return city;
    }
    public void setCity(CityDTO city) {
      this.city = city;
    }
    public Boolean getAllStews() {
      return allStews;
    }
    public void setAllStews(Boolean allStews) {
      this.allStews = allStews;
    }
    public String getMailText() {
      return mailText;
    }
    public void setMailText(String mailText) {
      this.mailText = mailText;
    }
    
    
  }
}
