package cz.sa.shiftPlanner.web.panels;

import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.TimeCodeDTO;
import cz.sa.shiftPlanner.entities.SymbolVO;

public class BusTimetablePanel extends Panel{

	
	public BusTimetablePanel(String id, List<BusTimetableDTO> lines){
		super(id);
		
		showOneDirection(lines);
	}
	
	private void showOneDirection(List<BusTimetableDTO> listOfTimetables) {

		RepeatingView rows = new RepeatingView("rows");
		add(rows);
		
		List<BusTimetableDTO> timetables = listOfTimetables;
		
		for(int i=0; i< 4; i++){
			AbstractItem item1 = new AbstractItem(rows.newChildId());
			
			RepeatingView cols = new RepeatingView("cols");
			item1.add(cols);
			
			switch(i){
				case 0:{
					AbstractItem emptyLeft = new AbstractItem(cols.newChildId());
					emptyLeft.add(new Label("label", ""));
					cols.add(emptyLeft);
					
					for (BusTimetableDTO busT : timetables) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
						item.add(new Label("label",printWeekDaysAsNumber(busT.getSymbols(), null)));
						cols.add(item);
					}
					rows.add(item1);
					break;
				}
				case 1:{
					AbstractItem emptyLeft = new AbstractItem(cols.newChildId());
					emptyLeft.add(new Label("label", ""));
					cols.add(emptyLeft);
					
					for (BusTimetableDTO busT : timetables) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
							if(busT.getTimeCode() != null)
								item.add(new Label("label", busT.getTimeCode().getMark())
									.add(new AttributeModifier("title", busT.getTimeCode().getDescription())));
							else
								item.add(new Label("label",""));
						cols.add(item);
					}
					rows.add(item1);
					break;
				}
				case 2 : {
					AbstractItem leftCities = new AbstractItem(cols.newChildId());
					leftCities.add(new Label("label", timetables.get(0).getDepartureStation().getCity().getCityName() + " "+
					                              timetables.get(0).getDepartureStation().getStationName().substring(0, 2))
								.add(new AttributeModifier("title", timetables.get(0).getDepartureStation().getStationName())));
					cols.add(leftCities);
					
					for (BusTimetableDTO busT : timetables) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
						item.add(new Label("label", dateFormat(busT.getDepartureTime())));
			
						cols.add(item);
					}
					rows.add(item1);
					break; 
				}	
				case 3:{
					AbstractItem leftCities = new AbstractItem(cols.newChildId());
					leftCities.add(new Label("label", timetables.get(0).getArrivalStation().getCity().getCityName()+ " "+ timetables.get(0).getArrivalStation().getStationName().substring(0, 2))
								.add(new AttributeModifier("title", timetables.get(0).getArrivalStation().getStationName())));
					cols.add(leftCities);
					
					for (BusTimetableDTO busT : timetables) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
						item.add(new Label("label", dateFormat(busT.getArrivalTime())));
			
						cols.add(item);
					}
					rows.add(item1);
					break;
				}
			}
		}
	}
	
	
	private String dateFormat(DateTime time){
		String sTime;
		sTime= " "+String.format("%02d",time.getHourOfDay()) +":"+ String.format("%02d",time.getMinuteOfHour())+" ";
		return sTime;
	}

	private String printWeekDaysAsNumber(List<SymbolVO> days, TimeCodeDTO timecode){
		String output = "";
		for(SymbolVO d : days){
			output += d.getMark();
			output += " ";
		}
		if(timecode != null)
			output += ("'"+timecode.getMark()+"'");
		return output;
	}
}
