package cz.sa.shiftPlanner.web.panels;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.BusTimetableService;
import cz.sa.shiftPlanner.services.BusTurnsTimetableService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.web.TimetableViewPage;
import cz.sa.shiftPlanner.web.wizard.TimetableTurnsWizard;

public class BusTurnsPanel extends Panel{

	@SpringBean(name = "busTimetableService")
	BusTimetableService busTimetableService;
	
	@SpringBean(name = "busTurnsTimetableService")
	BusTurnsTimetableService busTurnsTimetableService;
	
	private String [] colours = {"#FF66FF","#CCCCFF","#CC6666","#FFCC33","#FF6666","#FFFF33","#99CCFF","#99FF66","#CCFF99","#00FF99"};
	
	private IModel<List<BusTimetableDTO>> lines;
	private DateTime validity;
	private List<BusTurnsTimetableDTO> turns;
	
	public BusTurnsPanel(String id, IModel<List<BusTimetableDTO>> lines, WeekDay day, Integer week, DateTime validity){
		super(id);
		
		this.lines = lines;
		turns = new ArrayList<BusTurnsTimetableDTO>();
		this.validity = validity;
		showTimeCodes();
		showFormBoxes( day, week);
		showOneDirection();
	}
	
	private void showTimeCodes(){
		RepeatingView rows = new RepeatingView("codeRows");
		add(rows);
		
		AbstractItem item1 = new AbstractItem(rows.newChildId());
		
		RepeatingView cols = new RepeatingView("codeCols");
		item1.add(cols);
			
		//prazdne okno, pro spravne formatovani
		AbstractItem emptyLeft = new AbstractItem(cols.newChildId());
		emptyLeft.add(new Label("codeLabel", "").setVisible(false));
		cols.add(emptyLeft);
		
		for (BusTimetableDTO busT : lines.getObject()) {
			
			AbstractItem item = new AbstractItem(cols.newChildId());
				if(busT.getTimeCode() != null)
					item.add(new Label("codeLabel", busT.getTimeCode().getMark())
						.add(new AttributeModifier("title", busT.getTimeCode().getDescription())));
				else
					item.add(new Label("codeLabel", ""));
			cols.add(item);
		}
		rows.add(item1);
	}
	
	private void showFormBoxes( WeekDay day, Integer week) {
		Form form = new Form("form");
		add(form);
		
		RepeatingView rows = new RepeatingView("formRows");
		form.add(rows);
		
		
		AbstractItem item1 = new AbstractItem(rows.newChildId());
		
		RepeatingView cols = new RepeatingView("formCols");
		item1.add(cols);
		
		{
			AbstractItem emptyLeft = new AbstractItem(cols.newChildId());
			emptyLeft.add(new Label("formLabel", "").setVisible(false));
			cols.add(emptyLeft);
			
			for (BusTimetableDTO busT : lines.getObject()) {
				AbstractItem item = new AbstractItem(cols.newChildId());
				
				BusTurnsTimetableDTO turn;
				if(validity == null){
					turn = busTurnsTimetableService.findByDayAndTimetable(busT, day, week);
				}else{
					turn = busTurnsTimetableService.findByDayAndTimetable(busT, day, validity, DateFormatter.getWeekMod2(validity));
				}
				
				if(turn == null){
					turn  = new BusTurnsTimetableDTO();
					turn.setTimetable(busT);
					turn.setWeekDay(day);
					turn.setWeek(week);
					turn.setValidity_from(validity);
					turn.setValidity_to(validity);
					item.add(new TextField("formLabel", new PropertyModel(turn, "coupleNumber")));
					List<BusTurnsTimetableDTO> set = new ArrayList<BusTurnsTimetableDTO>();
					set.add(turn);
					busT.setTurns(set);
				}else{
					if(day.equals(WeekDay.WHOLEWEEK)) turn.setWeekDay(WeekDay.WHOLEWEEK);
					if(day.equals(WeekDay.WORKDAY)) turn.setWeekDay(WeekDay.WORKDAY);
					if(turn.getCoupleNumber()!=null)
						item.add(new TextField("formLabel", new PropertyModel<Integer>(turn, "coupleNumber"))
						.add(new AttributeModifier("style", "background-color:"+colours[turn.getCoupleNumber()%10]+";")));
					else
						item.add(new TextField("formLabel", new PropertyModel<Integer>(turn, "coupleNumber")));
				}
				turns.add(turn);
				
				cols.add(item);
			}
			rows.add(item1);
			
		}
	}
	
	/*
	 * vykresli nazvy mest a casy odjezdu
	 */
	private void showOneDirection() {
		
		
		RepeatingView rows = new RepeatingView("rows");
		add(rows);
		
		
		
		for(int i=0; i< 2; i++){
			AbstractItem item1 = new AbstractItem(rows.newChildId());
			
			RepeatingView cols = new RepeatingView("cols");
			item1.add(cols);
			
			switch(i){
				
				case 0 : {
					AbstractItem leftCities = new AbstractItem(cols.newChildId());
					leftCities.add(new Label("label", lines.getObject().get(0).getDepartureStation().getCity().getCityName() + " "+lines.getObject().get(0).getDepartureStation().getStationName().substring(0, 2)));
					cols.add(leftCities);
					
					for (BusTimetableDTO busT : lines.getObject()) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
						item.add(new Label("label", dateFormat(busT.getDepartureTime())));
			
						cols.add(item);
					}
					rows.add(item1);
					break; 
				}	
				case 1:{
					AbstractItem leftCities = new AbstractItem(cols.newChildId());
					leftCities.add(new Label("label", lines.getObject().get(0).getArrivalStation().getCity().getCityName() + " "+lines.getObject().get(0).getArrivalStation().getStationName().substring(0, 2)));
					cols.add(leftCities);
					
					for (BusTimetableDTO busT : lines.getObject()) {
						
						AbstractItem item = new AbstractItem(cols.newChildId());
						item.add(new Label("label", dateFormat(busT.getArrivalTime())));
			
						cols.add(item);
					}
					rows.add(item1);
					break;
				}
				
			}
		}
	}
	
	
	private String dateFormat(DateTime time){
		String sTime;
		sTime= String.format("%02d",time.getHourOfDay()) +" : "+ String.format("%02d",time.getMinuteOfHour());
		return sTime;
	}

	
	public void save(boolean manageBohtWeeks){
		List<BusTurnsTimetableDTO> sresult = new ArrayList<BusTurnsTimetableDTO>();
		List<BusTurnsTimetableDTO> rresult = new ArrayList<BusTurnsTimetableDTO>();
		for(BusTurnsTimetableDTO t : turns){
			if(t.getCoupleNumber() != null) 
				sresult.add(t); 
			if(t.getCoupleNumber() == null && t.getId() != null)
				rresult.add(t);
		}
		busTurnsTimetableService.removeList(rresult, manageBohtWeeks);
		busTurnsTimetableService.saveList(sresult, validity, manageBohtWeeks);
	}
}

