package cz.sa.shiftPlanner.web.panels;

import java.util.List;

import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.Page;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableLabel;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableMultiLineLabel;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow.WindowClosedCallback;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.services.BusConnectionService;
import cz.sa.shiftPlanner.services.BusTurnsTimetableService;
import cz.sa.shiftPlanner.services.WayService;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.modalWindow.WayPersonModal;

public class BusWaysPanel extends Panel {

  @SpringBean(name = "busConnectionService")
  BusConnectionService busConnectionService;

  @SpringBean(name = "wayService")
  WayService wayService;

  @SpringBean(name = "busTurnsTimetableService")
  BusTurnsTimetableService busTurnsTimetableService;
  
  private static final Logger log = Logger.getLogger(BusWaysPanel.class);

  private IModel<List<BusConnectionDTO>> lines;
  private BusTurnsTimetableDTO turnB;
  private BusTurnsTimetableDTO turnA;
  private final ModalWindow addModalWindow;
  private IModel<PersonDTO> person;
  private WebMarkupContainer container;
  private boolean isAdmin = false;

  public BusWaysPanel(String id, IModel<List<BusConnectionDTO>> lines, IModel<PersonDTO> person, WebMarkupContainer container) {
    super(id);

    this.lines = lines;
    this.container = container;
    this.person = person;
    turnB = lines.getObject().get(0).getBusTurn();
    turnA = lines.getObject().get(lines.getObject().size() - 1).getBusTurn();

    if (PlannerSession.get().getRoles().contains(PlannerRoles.COORDINATOR.toLowerCase())
        || PlannerSession.get().getRoles().contains(PlannerRoles.SUPERVISOR.toLowerCase())) {
      isAdmin = true;
    }

    add(addModalWindow = new ModalWindow("modal"));
    addModalWindow.setCookieName("modal");
    addModalWindow.setWindowClosedCallback(new PWindowClosedCallback(container));

    drawComponent();
  }

  private class PWindowClosedCallback implements WindowClosedCallback {

    private Component componentToUpdate;

    public PWindowClosedCallback(Component componentToUpdate) {
      this.componentToUpdate = componentToUpdate;
    }

    @Override
    public void onClose(AjaxRequestTarget target) {
      target.add(componentToUpdate);
    }

  }

  /////////////////////////////////////////////////////////
  private void drawComponent() {

    final Form form = new Form("form");
    form.setOutputMarkupId(true);
    add(form);
    RepeatingView rows = new RepeatingView("rows");

    form.add(rows);

    Label busNumber = new Label("busNumber", "BUS " + new PropertyModel<Integer>(turnB, "coupleNumber").getObject());
    MetaDataRoleAuthorizationStrategy.authorize(busNumber, ENABLE, PlannerRoles.COORDINATOR);
    form.add(busNumber);

    AjaxEditableMultiLineLabel noteBefore = new AjaxEditableMultiLineLabel("noteBefore", new PropertyModel(turnB, "noteBefore")){
      @Override
      protected void onSubmit(AjaxRequestTarget target) {
        super.onSubmit(target);
        busTurnsTimetableService.update(turnB);
      }
    };
    MetaDataRoleAuthorizationStrategy.authorize(noteBefore, ENABLE, PlannerRoles.COORDINATOR);
    form.add(noteBefore);

    AjaxEditableMultiLineLabel noteAfter = new AjaxEditableMultiLineLabel("noteAfter", new PropertyModel(turnA, "noteAfter")){
      @Override
      protected void onSubmit(AjaxRequestTarget target) {
        super.onSubmit(target);
        busTurnsTimetableService.update(turnA);
      }
    };
    MetaDataRoleAuthorizationStrategy.authorize(noteAfter, ENABLE, PlannerRoles.COORDINATOR);
    form.add(noteAfter);

    AjaxCheckBox checkAll = new AjaxCheckBox("checkAll", Model.of(Boolean.FALSE)) {

      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        try {
          wayService.createAll(lines.getObject(), person.getObject(), false, isAdmin);
        }
        catch (ServiceNoticeException e) {
          container.error(e.getMessage());
        }

        target.add(container);
      }
    };
    MetaDataRoleAuthorizationStrategy.authorize(checkAll, RENDER, PlannerRoles.STEWARD);

    form.add(checkAll);
    AjaxLink removeAll = new AjaxLink("removeAll") {
      @Override
      public void onClick(AjaxRequestTarget target) {
        for (BusConnectionDTO busT : lines.getObject()) {
          if (busT.getWays().size() == 0)
            continue;
          if (busT.getWays().size() == 1) {
            wayService.remove(busT.getWays().get(0));
          }
          else if (busT.getWays().size() == 2) {
            for (WayDTO w : busT.getWays()) {
              wayService.remove(w);
            }
          }
          busT.getWays().clear();
        }
        target.add(container);

      }
    };
    MetaDataRoleAuthorizationStrategy.authorize(removeAll, RENDER, PlannerRoles.COORDINATOR);
    form.add(removeAll);

    for (int i = 0; i < 4; i++) {
      AbstractItem rowsItem = new AbstractItem(rows.newChildId());

      RepeatingView cols = new RepeatingView("cols");
      rowsItem.add(cols);

      switch (i) {
        case 0: {

          for (final BusConnectionDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new AttributeModifier("colspan", 2));

            AjaxEditableLabel wayNote = new AjaxEditableLabel("fragmentWrap", new PropertyModel(busT, "note")) {
              @Override
              protected void onSubmit(AjaxRequestTarget target) {
                super.onSubmit(target);
                busConnectionService.update(busT);
              }
              @Override
              protected String defaultNullLabel() {
                return "-";
              }
            };

            MetaDataRoleAuthorizationStrategy.authorize(wayNote, ENABLE, PlannerRoles.COORDINATOR);
            colsItem.add(wayNote);
            cols.add(colsItem);

          }
          rows.add(rowsItem);
          break;
        }

        case 1: {

          for (BusConnectionDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());

            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this,
                 busT.getTimetable().getDepartureStation().getCity().getCityName()).add(new AttributeModifier("title", busT.getTimetable()
                .getDepartureStation().getStationName())));
            cols.add(colsItem);

            colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this,
                 busT.getTimetable().getArrivalStation().getCity().getCityName()).add(new AttributeModifier("title", busT.getTimetable()
                .getArrivalStation().getStationName())));

            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
        case 2: {

          for (BusConnectionDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());

            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this, dateFormatToTime(busT.getDepartureTime())));
            cols.add(colsItem);

            colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this, dateFormatToTime(busT.getArrivalTime())));

            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
        case 3: {

          for (final BusConnectionDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new AttributeModifier("colspan", 2));

            Fragment fragment;

            AjaxLink delete = new AjaxLink("crossLink") {
              @Override
              public void onClick(AjaxRequestTarget target) {
                wayService.remove(busT.getWays().get(0));
                busT.getWays().clear();
                target.add(container);
              }
            };

            if (busT.getDepartureTime().compareTo(new DateTime().minusDays(1)) < 0) {
              delete.setVisible(false);
            }

            MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
//   Nevim k cemu to bylo, ale zda se ze je to stara funkcionalita, jednalo se o porovnavani na zaklade datecreated
//             Collections.sort(busT.getWays(), new WayDTOComparator());
             final List<WayDTO> ways = busT.getWays();

            if (ways == null || ways.size() == 0 ) {
              fragment = new Fragment("fragmentWrap", "checkFragment", this);
              CheckBox box = new CheckBox("checkbox", Model.of(Boolean.FALSE));
              box.add(new AjaxFormComponentUpdatingBehavior("click") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                  WayDTO way;
                  try {
                    way = wayService.create(busT, person.getObject(), false, isAdmin);
                    busT.getWays().add(way);
                  }
                  catch (ServiceNoticeException e) {
                    container.error(e.getMessage());
                  }
                  target.add(container);
                }
              });
              fragment.add(box);
              MetaDataRoleAuthorizationStrategy.authorize(fragment, RENDER, PlannerRoles.STEWARD);
              colsItem.add(new AttributeModifier("style", "background-color:#FFCCCC;"));
            }
            else {
              if (busT.getWays().size() == 1) {

                fragment = new MyFragment("fragmentWrap", "labelFragment", this, ways.get(0).getPerson().getSurname() + " "
                    + ways.get(0).getPerson().getName());
                fragment.add(delete);

                addModalWindow.setPageCreator(new ModalWindow.PageCreator() {
                  public Page createPage() {
                    return new WayPersonModal(ways.get(0), addModalWindow);
                  }
                });

                AjaxLink addLink = new AjaxLink<Void>("showModal") {
                  @Override
                  public void onClick(AjaxRequestTarget target) {
                    addModalWindow.show(target);
                  }
                };
                MetaDataRoleAuthorizationStrategy.authorize(addLink, RENDER, PlannerRoles.COORDINATOR);
                fragment.add(addLink);

              }
              else {
                fragment = new MyComplexeFragment("fragmentWrap", "complexeFragment", this, ways);
              }
            }
            colsItem.add(fragment);

            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
      }
    }
  }

  private class MyFragment extends Fragment {

    public MyFragment(String id, String markupId, MarkupContainer markupProvider, String string) {
      super(id, markupId, markupProvider);

      if (markupId.equals("labelFragment")) {

        add(new Label("label", string));
      }
      if (markupId.equals("textFragment")) {

        add(new Label("label", string));
      }
    }
  }

  private class MyComplexeFragment extends Fragment {

    public MyComplexeFragment(String id, String markupId, MarkupContainer markupProvider, final List<WayDTO> ways) {
      super(id, markupId, markupProvider);

      AjaxLink delete = new AjaxLink("crossLink") {
        @Override
        public void onClick(AjaxRequestTarget target) {
          //odstranim cestu na ktere se tyka krizek
          wayService.remove(ways.get(0));

          //fiktivne odstranim i tu druhou a pote ji znovu vytvorim s hodnotami prislusne bus_connection
          BusConnectionDTO conn = ways.get(1).getConnection();
          PersonDTO per = ways.get(1).getPerson();
          wayService.remove(ways.get(1));
          WayDTO newWay;
          try {
            newWay = wayService.create(conn, per, false, true);
            ways.clear();
            ways.add(newWay);
          }
          catch (ServiceNoticeException e) {
            log.error("Nastal problem pri pridavani casu zbyvajicimu stevardovi.", e);
            container.error("Nastal problem pri pridavani casu zbyvajicimu stevardovi.");
          }
          target.add(container);
        }
      };

      AjaxLink delete1 = new AjaxLink("crossLink1") {
        @Override
        public void onClick(AjaxRequestTarget target) {
         //odstranim cestu na ktere se tyka krizek
          wayService.remove(ways.get(1));

          //fiktivne odstranim i tu druhou a pote ji znovu vytvorim s hodnotami prislusne bus_connection
          BusConnectionDTO conn = ways.get(0).getConnection();
          PersonDTO per = ways.get(0).getPerson();
          wayService.remove(ways.get(0));
          WayDTO newWay;
          try {
            newWay = wayService.create(conn, per, false, true);
            ways.clear();
            ways.add(newWay);
          }
          catch (ServiceNoticeException e) {
            log.error("Nastal problem pri pridavani casu zbyvajicimu stevardovi.", e);
            container.warn("Nastal problem pri pridavani casu zbyvajicimu stevardovi.");
          }
          target.add(container);
        }
      };
      MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
      MetaDataRoleAuthorizationStrategy.authorize(delete1, RENDER, PlannerRoles.COORDINATOR);

      add(new Label("labelFrom", ways.get(0).getDepartureStation().getCity().getCityName().substring(0, 3) + " - "
          + ways.get(0).getArrivalStation().getCity().getCityName().substring(0, 3)).add(new AttributeModifier("title", "Z: "
          + ways.get(0).getDepartureStation().getCity().getCityName() + " Do: " + ways.get(0).getArrivalStation().getCity().getCityName())));
      add(new Label("labelTo", ways.get(1).getDepartureStation().getCity().getCityName().substring(0, 3) + " - "
          + ways.get(1).getArrivalStation().getCity().getCityName().substring(0, 3)).add(new AttributeModifier("title", "Z: "
          + ways.get(1).getDepartureStation().getCity().getCityName() + " Do: " + ways.get(1).getArrivalStation().getCity().getCityName())));
      add(new Label("labelP", ways.get(0).getPerson().getSurname()));
      add(delete);
      add(new Label("labelP1", ways.get(1).getPerson().getSurname()));
      add(delete1);
    }
  }

  private String dateFormatToTime(DateTime time) {
    String sTime;
    sTime = String.format("%02d", time.getHourOfDay()) + " : " + String.format("%02d", time.getMinuteOfHour());
    return sTime;
  }

}
