package cz.sa.shiftPlanner.web.panels;

import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;

import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.pcomponent.PFeedbackPanel;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;

/**
 * @author ToSk
 */

public class CentralRulesPanel extends Panel {

  @SpringBean(name = "personService")
  PersonService personService;

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;

  private WebMarkupContainer listContainer;
  private IModel<PersonDTO> personModel;
  private List<PersonDTO> categories;

  public CentralRulesPanel(String id, IModel<List<List<BusTurnsTimetableDTO>>> timetables,  WeekDay day, Integer week, PageParameters parameters) {
    super(id);

    listContainer = new WebMarkupContainer("container");
    personModel = new Model<PersonDTO>();
    listContainer.setOutputMarkupId(true);

    categories = personService.findByLineGroup(parameters.get("id").toLong());
    personModel.setObject(categories.get(0));
    Form<PersonDTO> personForm = new Form<PersonDTO>("personForm");
    final DropDownChoice<PersonDTO> choice = new DropDownChoice<PersonDTO>("personSelect", personModel, categories);
    choice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        target.add(listContainer);
      }
    });
    personForm.add(choice);
    listContainer.add(personForm);
    
    add(new Label("day", new ResourceModel("day."+day)));
    add(new Label("week",  " - tyden " + week));

    //repeater pro vykresleni panelu podle tocek
    LineViews views = new LineViews("times", timetables, day, personModel, week);

    listContainer.add(new PFeedbackPanel("feedbackPanel").setOutputMarkupId(true));
    listContainer.add(views);
    add(listContainer);

  }

  private class LineViews extends ListView<List<BusTurnsTimetableDTO>>{
    
    private WeekDay day;
    IModel<PersonDTO> person;
    Integer week;
    
  public LineViews(String id, IModel<List<List<BusTurnsTimetableDTO>>> model, WeekDay day, IModel<PersonDTO> person, Integer week) {
        super(id, model);
        this.day = day;
        this.person = person;
        this.week = week;
   }

  @Override
  protected void populateItem(ListItem <List<BusTurnsTimetableDTO>> item) {
    BusRulesPanel panel = new BusRulesPanel("lines",item.getModel(), day, week, person, listContainer);
    item.add(panel);
  }
}

}
