package cz.sa.shiftPlanner.web.panels.personDetail;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.extensions.yui.calendar.DateTimeField;
import org.apache.wicket.extensions.yui.calendar.TimeField;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.RefreshingView;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.ComponentPropertyModel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.ExcuseDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.enums.ExcuseReason;
import cz.sa.shiftPlanner.enums.ExcuseStatus;
import cz.sa.shiftPlanner.pcomponent.EnumDropDownChoice;
import cz.sa.shiftPlanner.pcomponent.PFeedbackPanel;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.PersonDetailPage;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.CalendarPanel;

/**
 * @author ToSk
 */

public class AbsencePanel extends Panel {

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;

  private final WebMarkupContainer viewContainer;
  private LoadableDetachableModel<List<ExcuseDTO>> exModel;
  private Form form;
  private PersonDTO person;
  private FeedbackPanel feedback;

  public AbsencePanel(String id, final PersonDTO person) {
    super(id);
    this.person = person;

    feedback = new PFeedbackPanel("feedbackPanel");
    feedback.setOutputMarkupId(true);
    viewContainer = new WebMarkupContainer("container");
    viewContainer.setOutputMarkupId(true);
    add(viewContainer);
    viewContainer.add(new Label("message", "Absence"), feedback);

    //nacitani absenci
    exModel = new LoadableDetachableModel<List<ExcuseDTO>>() {
      @Override
      protected List<ExcuseDTO> load() {
        return personUtilsService.findAllByPersonId(person.getId());
      }
    };

    //create instance excuse form
    viewContainer.add(form = new ExForm("excuseForm", new ExcuseDTO()));
    MetaDataRoleAuthorizationStrategy.authorize(form, RENDER, PlannerRoles.STEWARD);
    renderViewTable();
  }

  private class ExForm extends Form<ExcuseDTO> {
    
    private IModel<Date> dateModelFrom;
    private IModel<Date> dateModelTo = new Model<Date>();
    private List<ExcuseReason> choices = Arrays.asList(ExcuseReason.values());
    private TextArea<String> textArea;

    public ExForm(String id, ExcuseDTO ex) {
      super(id, new CompoundPropertyModel<ExcuseDTO>(ex));

      add(new AjaxButton("ajaxButton") {
        @Override
        protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
          //naplneni excuse daty ktera se nenaplni automaticky z compoundPropertyModelu
          ExcuseDTO ex = ExForm.this.getModelObject();
          ex.setPerson(person);
          ex.setFrom_(new DateTime(dateModelFrom.getObject()));
          ex.setTo_(new DateTime(dateModelTo.getObject()));
          if (ex.getId() == null){
            if(PlannerSession.get().getLoggedUser().getRole().equalsIgnoreCase(PlannerRoles.STEWARD)){
              ex.setStatus(ExcuseStatus.REQUESTED);
              personUtilsService.askExcuse(ex);
            }else{
              ex.setStatus(ExcuseStatus.APPROVED);
              personUtilsService.createExcuse(ex);
            }
          }else
            personUtilsService.updateExcuse(ex, ex.getStatus());
          //reset formulare
          ExForm.this.setModelObject(new ExcuseDTO());
          target.add(viewContainer);
        }
        
        @Override
        public void onError(AjaxRequestTarget target, Form<?> form) {
          target.add(feedback);
        }
      });

      add(new Label("person_name", new PropertyModel<String>(person, "name")));
      add(new Label("person_surname", new PropertyModel<String>(person, "surname")));

      add(new EnumDropDownChoice<ExcuseReason>("reason", choices).setRequired(true));

      //nastaveni modelu tak aby vytvarel novou instanci date nebo nacital existujici pri editovani
      dateModelFrom = new LoadableDetachableModel<Date>() {
        @Override
        protected Date load() {
          if (getModelObject().getFrom_() == null) {
            Date da = new Date();
            da.setHours(0);
            da.setMinutes(0);
            return da;
          }
          return getModelObject().getFrom_().toDate();
        }
      };

      DateTimeField calendar = new DateTimeField("calendar", dateModelFrom);
      calendar.setRequired(true);
      add(calendar);

      dateModelTo = new LoadableDetachableModel<Date>() {
        @Override
        protected Date load() {
          if (getModelObject().getFrom_() == null) {
            Date d = new Date();
            d.setHours(23);
            d.setMinutes(59);
            return d;
          }
          return getModelObject().getTo_().toDate();
        }
      };

      DateTimeField calendar2 = new DateTimeField("calendar2", dateModelTo);
      calendar2.setRequired(true);
      add(calendar2);

      add(textArea = new TextArea<String>("excuseText"));
    }

  }

  private void renderViewTable() {
    Component header = new AbstractItem("header");
    ListView<ExcuseDTO> repeatingView = new ListView<ExcuseDTO>("repeating", exModel) {

      @Override
      protected void populateItem(final ListItem<ExcuseDTO> item) {

        item.add(new Label("dateFrom", String.valueOf(DateFormatter.toCompletTimestamp(item.getModelObject().getFrom_()))));
        item.add(new Label("dateTo", String.valueOf(DateFormatter.toCompletTimestamp(item.getModelObject().getTo_()))));
        item.add(new Label("reason", new ResourceModel("absence.reason." +item.getModelObject().getReason())));
        item.add(new Label("text", item.getModelObject().getExcuseText() == null ? "" : item.getModelObject().getExcuseText()));
        item.add(new Label("status", new ResourceModel("absence.status." +item.getModelObject().getStatus())));

        AjaxLink approve = new AjaxLink("approveLink") {
          @Override
          public void onClick(AjaxRequestTarget target) {
            personUtilsService.updateExcuse(item.getModelObject(), ExcuseStatus.APPROVED);
            exModel.detach();
            target.add(viewContainer);
          }
        };
        
        AjaxLink reject = new AjaxLink("rejectLink") {
          @Override
          public void onClick(AjaxRequestTarget target) {
            personUtilsService.updateExcuse(item.getModelObject(), ExcuseStatus.REJECTED);
            exModel.detach();
            target.add(viewContainer);
          }
        };
        
        AjaxLink delete = new AjaxLink("crossLink") {
          @Override
          public void onClick(AjaxRequestTarget target) {
            personUtilsService.remove(item.getModelObject());
            exModel.detach();
            target.add(viewContainer);
          }
        };

        AjaxLink addLink = new AjaxLink<Void>("addLink") {
          @Override
          public void onClick(AjaxRequestTarget target) {
            form.setModelObject(item.getModelObject());
            target.add(viewContainer);
          }
        };
        MetaDataRoleAuthorizationStrategy.authorize(addLink, RENDER, PlannerRoles.COORDINATOR);
        MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
        MetaDataRoleAuthorizationStrategy.authorize(reject, RENDER, PlannerRoles.COORDINATOR);
        MetaDataRoleAuthorizationStrategy.authorize(approve, RENDER, PlannerRoles.COORDINATOR);
        if(item.getModelObject().getStatus().equals(ExcuseStatus.APPROVED)){
          reject.setVisible(false);
          approve.setVisible(false);
        }
        if(item.getModelObject().getStatus().equals(ExcuseStatus.REJECTED)){
          reject.setVisible(false);
        }
        item.add(delete, addLink, reject, approve);

        item.add(new AttributeModifier("class", (item.getIndex() % 2 == 1) ? "even" : "odd"));
      }
    };
    viewContainer.add(header, repeatingView);

  }

  @Override
  protected void detachModel() {
    super.detachModel();
    exModel.detach();
  }

}
