package cz.sa.shiftPlanner.web.panels.personDetail;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.extensions.yui.calendar.DateTimeField;
import org.apache.wicket.extensions.yui.calendar.TimeField;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.RefreshingView;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.ComponentPropertyModel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.ExcuseDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.PersonRecordDTO;
import cz.sa.shiftPlanner.enums.ExcuseReason;
import cz.sa.shiftPlanner.enums.MeasurementType;
import cz.sa.shiftPlanner.pcomponent.EnumDropDownChoice;
import cz.sa.shiftPlanner.pcomponent.PFeedbackPanel;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.PersonDetailPage;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.CalendarPanel;

/**
 * @author ToSk
 */

public class NotesPanel extends Panel {

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;

  private final WebMarkupContainer viewContainer;
  private IModel<List<PersonRecordDTO>> noteModel;
  private Form form;
  private PersonDTO person;
  private FeedbackPanel feedback;

  public NotesPanel(String id, final PersonDTO person) {
    super(id);
    this.person = person;
    
    feedback = new PFeedbackPanel("feedbackPanel");
    feedback.setOutputMarkupId(true);
    viewContainer = new WebMarkupContainer("container");
    viewContainer.setOutputMarkupId(true);
    add(viewContainer);
    viewContainer.add(new Label("message", "Poznamky"), feedback);

    //nacitani zaznamu
    noteModel = new LoadableDetachableModel<List<PersonRecordDTO>>() {
      @Override
      protected List<PersonRecordDTO> load() {
        return personUtilsService.findRecordsByPerson(person);
      }
    };

    //create instance excuse form
    viewContainer.add(form = new NtForm("excuseForm", new PersonRecordDTO()));
    MetaDataRoleAuthorizationStrategy.authorize(form, RENDER, PlannerRoles.COORDINATOR);
    renderViewTable();
  }

  private class NtForm extends Form<PersonRecordDTO> {

    @SpringBean(name = "personService")
    PersonService personService;

    private List<MeasurementType> choices = Arrays.asList(MeasurementType.values());
    private TextArea<String> textArea;

    public NtForm(String id, PersonRecordDTO rec) {
      super(id, new CompoundPropertyModel<PersonRecordDTO>(rec));

      add(new AjaxButton("ajaxButton") {
        @Override
        protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
          PersonRecordDTO record = NtForm.this.getModelObject();
          record.setPerson(person);
          PersonDTO createdBy = ((PlannerSession)PlannerSession.get()).getLoggedUser().getPerson();
          record.setCreatedBy(createdBy);
          personUtilsService.createRecord(record);
          target.add(viewContainer);
        }
        
        @Override
        public void onError(AjaxRequestTarget target, Form<?> form) {
          target.add(feedback);
        }
      });

      add(new Label("person_name", new PropertyModel<String>(person, "name")));
      add(new Label("person_surname", new PropertyModel<String>(person, "surname")));

      add(new EnumDropDownChoice<MeasurementType>("type", choices).setRequired(true));

      add(textArea = new TextArea<String>("description"));
      textArea.setRequired(true);
    }

  }

  private void renderViewTable() {

    ListView<PersonRecordDTO> repeatingView = new ListView<PersonRecordDTO>("repeating", noteModel) {

      @Override
      protected void populateItem(final ListItem<PersonRecordDTO> item) {

        item.add(new Label("type", new ResourceModel("MeasurementType." + item.getModelObject().getType())));
        item.add(new Label("created", item.getModelObject().getDateCreated()));
        item.add(new Label("text", item.getModelObject().getDescription() == null ? "" : item.getModelObject().getDescription()));
        item.add(new Label("createdBy", item.getModelObject().getCreatedBy()));
  
        AjaxLink delete = new AjaxLink("crossLink") {
          @Override
          public void onClick(AjaxRequestTarget target) {
            personUtilsService.removeRecord(item.getModelObject());
            noteModel.detach();
            target.add(viewContainer);
          }
        };
        MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
        item.add(delete);

        item.add(new AttributeModifier("class", (item.getIndex() % 2 == 1) ? "even" : "odd"));
      }
    };
    viewContainer.add( repeatingView);

  }

  @Override
  protected void detachModel() {
    super.detachModel();
    noteModel.detach();
  }

}
