package cz.sa.shiftPlanner.web.vacation;

import java.text.DateFormat;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.model.StringResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.ExcuseDTO;
import cz.sa.shiftPlanner.enums.ExcuseReason;
import cz.sa.shiftPlanner.pcomponent.EnumDropDownChoice;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.web.MenuPage;
import cz.sa.shiftPlanner.web.PersonDetailPage;
import cz.sa.shiftPlanner.web.panels.personDetail.WaysViewPanel;

/**
 *
 * @author    ToSk
 * @version   1.0
 * @created   8. 10. 2013
 */

public class VacationViewPage extends MenuPage {

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;
  
  private IModel<String> monthModel = new Model<String>();
  private WebMarkupContainer vacationContainer;
  private List<String> czMonths;
  private IModel<ExcuseReason> reasonModel = new Model<ExcuseReason>();
  
  public VacationViewPage(PageParameters parameters) {
    super(parameters);

    add(vacationContainer = new WebMarkupContainer("vacationContainer"));
    vacationContainer.setOutputMarkupId(true);

    renderMonthChoice();
    renderReasons();
    renderDaysWithContent();
  }

  private void renderDaysWithContent() {
    RepeatingView rows = new RepeatingView("rows");
    RepeatingView cols= new RepeatingView("cols");
    AbstractItem rowsItem = new AbstractItem(rows.newChildId());
    AbstractItem colsItem;
    vacationContainer.addOrReplace(rows);
    
    rowsItem.add(cols);
    rows.add(rowsItem);
    DateTime toDays = new DateTime().withDayOfWeek(1);
     for (int i = 0; i < 7; i++){
       colsItem = new AbstractItem(cols.newChildId());
       colsItem.add(new Label("fragmentWrap", toDays.dayOfWeek().getAsText(new Locale("cs", "CZ"))));
       cols.add(colsItem);
       toDays = toDays.plusDays(1);
     }
    
    DateTime from = DateFormatter.getBeginOfMonthThisYear(czMonths.indexOf(monthModel.getObject())+1);
    DateTime to   = DateFormatter.getEndOfMonthThisYear(czMonths.indexOf(monthModel.getObject())+1);
    
    int i = 1;
    while (from.compareTo(to) <= 0) {
      if(i %7 ==1){
          rowsItem = new AbstractItem(rows.newChildId());
          cols= new RepeatingView("cols");
          rowsItem.add(cols);
          rows.add(rowsItem);
          if(from.getDayOfMonth() == 1){
            //vykreslim tolik prazdnych poli, kolik je rozdil mezi prvnim dnem v mesici a aktualnim dnem
            for(int j =1; j< from.getDayOfWeek(); j++){
              colsItem = new AbstractItem(cols.newChildId());
              colsItem.add(new EmptyFragment("fragmentWrap", "emptyFragment", this));
              cols.add(colsItem);
              i++;
            }
          }
      }
      colsItem = new AbstractItem(cols.newChildId());
      colsItem.add(new ContentFragment("fragmentWrap", "vacationFragment", this, from));
      cols.add(colsItem);
      from = from.plusDays(1);
      i++;
    }
  }

  private void renderMonthChoice() {
    
    Locale locale = new Locale("cs", "CZ");
    czMonths = Lists.newArrayList(new DateFormatSymbols(locale).getMonths());
    monthModel.setObject(new DateTime().monthOfYear().getAsText(locale));
    DropDownChoice choice = new DropDownChoice("months", monthModel, Lists.newArrayList(czMonths));
    choice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
      
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        renderDaysWithContent();
        target.add(vacationContainer);
      }
    });
    add(choice);
  }
  
  private void renderReasons(){
    DropDownChoice choice =new EnumDropDownChoice<ExcuseReason>("reason", Arrays.asList(ExcuseReason.values()), reasonModel);
    choice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
      
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        renderDaysWithContent();
        target.add(vacationContainer);
      }
    });
    add(choice);
  }
  
  private class ContentFragment extends Fragment {

    private WebMarkupContainer content;
    
    public ContentFragment(String id, String markupId, MarkupContainer markupProvider, final DateTime dt) {
      super(id, markupId, markupProvider);
      add(content = new WebMarkupContainer("vacationContent"));
      
      if(dt.getDayOfMonth() == new DateTime().getDayOfMonth())
        content.add(AttributeModifier.append("style", "background-color: #F0F0F0;"));
      content.add(new Label("date", dt.getDayOfMonth()));
      IModel<List<ExcuseDTO>> excusesModel = new LoadableDetachableModel<List<ExcuseDTO>>() {
        @Override
        protected List<ExcuseDTO> load() {
          return personUtilsService.excusesInDay(dt, reasonModel.getObject());
        }
      };
      ListView<ExcuseDTO> listview = new ListView<ExcuseDTO> ("persons", excusesModel) {
        @Override
        protected void populateItem(ListItem<ExcuseDTO> item) {
          final PageParameters pageParameters = new PageParameters();
          pageParameters.add("perId", String.valueOf(item.getModelObject().getPerson().getId()));
          BookmarkablePageLink link = new BookmarkablePageLink("personLink", PersonDetailPage.class, pageParameters);
          ExcuseDTO ex = item.getModelObject();
          link.add(new Label("person", new StringResourceModel("absence.reason." +ex.getReason(), null, ex.getPerson())));
          link.add(new AttributeModifier("class", String.valueOf(ex.getReason()).toLowerCase()));
          item.add(link);
        }
      };
      content.add(listview);
    }
  }

  private class EmptyFragment extends Fragment {

    public EmptyFragment(String id, String markupId, MarkupContainer markupProvider) {
      super(id, markupId, markupProvider);
    }
  }
}
