package cz.sa.shiftPlanner.web.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.extensions.wizard.Wizard;
import org.apache.wicket.extensions.wizard.WizardModel;
import org.apache.wicket.extensions.wizard.WizardStep;
import org.apache.wicket.markup.html.form.CheckBoxMultipleChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.pcomponent.EnumCheckBoxMultipleChoice;
import cz.sa.shiftPlanner.services.BusTurnsTimetableService;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.web.MenuPage;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.BusRulesPanel;
import cz.sa.shiftPlanner.web.panels.CentralRulesPanel;

public class RulesWizard extends Wizard{
	
	@SpringBean(name = "lineGroupService")
	LineGroupService lineService;
	
	private PageParameters parameters;
	
	private IModel<ArrayList<WeekDay>> daysModel;
	private IModel<ArrayList<Integer>> weekModel;
	
	private List<BusRulesPanel> panels= new ArrayList<BusRulesPanel>();

	public RulesWizard(String id, PageParameters parameters) {
		super(id);
		
		this.parameters = parameters;
		daysModel = new Model<ArrayList<WeekDay>>(new ArrayList<WeekDay>());
		weekModel = new Model<ArrayList<Integer>>();
		
		 WizardModel model = new WizardModel();
	     model.add(new Days(daysModel, weekModel));
	     model.add(new Rules());

	     init(model);
	}

    
    private final class Days extends WizardStep{
    	
    	
       private List<Integer> weekChoices = new ArrayList<Integer>(Arrays.asList(1,2,3,4));
       private List<WeekDay> choices = new ArrayList<WeekDay>(Arrays.asList(WeekDay.values()));
       
        public Days(final IModel<ArrayList<WeekDay>> daysModel, final IModel<ArrayList<Integer>> weekModel){
        	super("","Pravidla pro linku: "+lineService.get(parameters.get("id").toLong()).getName());
        	
        	add(new FeedbackPanel("feedback"));
        	
        	Form weekForm = new Form("weekForm");
            
        	if(lineService.get(parameters.get("id").toLong()).isInterstate()){
        		weekChoices.remove(new Integer(3));
        		weekChoices.remove(new Integer(4));
        	}
    		weekForm.add(new CheckBoxMultipleChoice<Integer>("dayTypes", weekModel, weekChoices));
    		
    		add(weekForm);
    		
            Form form = new Form("dayForm");
            
            // v planovani se uz tyto dva zaznamy nevyskytuji, protoze stevardka se celemu tydnu naraz neprirazuje
            choices.remove(WeekDay.WORKDAY);
            choices.remove(WeekDay.WHOLEWEEK);
    		form.add(new EnumCheckBoxMultipleChoice<WeekDay>("dayTypes", daysModel, choices));
    		
    		add(form);
        }
        
    }

    private final class Rules extends WizardStep {
        
    	@SpringBean(name = "personService")
    	PersonService personService;
    	
    	@SpringBean(name = "busTurnsTimetableService")
    	BusTurnsTimetableService busTurnsTimetableService;
    	
    	List<IModel<List< List<BusTurnsTimetableDTO>>>> models = new ArrayList<IModel<List<List<BusTurnsTimetableDTO>>>>();
    	
    	public Rules(){
    	  super("","Pravidla pro linku: "+lineService.get(parameters.get("id").toLong()).getName());
    	}
    	
    	@Override
      protected void onBeforeRender() {
        super.onBeforeRender();
        createPanels(daysModel, parameters, weekModel);
      }

  		private void createPanels(final IModel<ArrayList<WeekDay>> daysModel, final PageParameters parameters, final IModel<ArrayList<Integer>> weekModel) {
          	
  		  List<ITab> tabs = new ArrayList<ITab>();
  
        
        for(final WeekDay day : daysModel.getObject()) {
          final IModel<List< List<BusTurnsTimetableDTO>>> timetableModel = new LoadableDetachableModel<List<List<BusTurnsTimetableDTO>>>() {
            @Override
            protected  List<List<BusTurnsTimetableDTO>> load(){
              return Lists.newArrayList(busTurnsTimetableService.findByLineGroupIdAndDay(parameters.get("id").toLong(), day, weekModel.getObject().get(0),
                  PlannerSession.get().getSelectedValidity()));
            }
            @Override
            public void detach() {
            }
          };
          
           tabs.add(new AbstractTab(new Model<String>(day.toString())) {
            public Panel getPanel(String panelId) {
              return new CentralRulesPanel(panelId, timetableModel, day, weekModel.getObject().get(0), parameters);
            }
          });
           models.add(timetableModel);
        }
        addOrReplace(new AjaxTabbedPanel("tabs", tabs));
  		}
  		
  		@Override
  		protected void detachModel() {
  		  super.detachModel();
  		  for (IModel<List< List<BusTurnsTimetableDTO>>> model : models) {
          model.detach();
        }
  		}
   }

    
    @Override
    public void onCancel()
    {
    	panels = new ArrayList<BusRulesPanel>();
        setResponsePage(MenuPage.class);
    }

   
    @Override
    public void onFinish()
    {
    	panels = new ArrayList<BusRulesPanel>();
        setResponsePage(WizardPage.class,parameters);
    }


	public LineGroupService getLineService() {
		return lineService;
	}


	public void setLineService(LineGroupService lineService) {
		this.lineService = lineService;
	}


	public PageParameters getParameters() {
		return parameters;
	}


	public void setParameters(PageParameters parameters) {
		this.parameters = parameters;
	}


	public List<BusRulesPanel> getPanels() {
		return panels;
	}


	public void setPanels(List<BusRulesPanel> panels) {
		this.panels = panels;
	}
    
    

}
