package cz.sa.shiftPlanner.web.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.wicket.extensions.wizard.IWizardStep;
import org.apache.wicket.extensions.wizard.Wizard;
import org.apache.wicket.extensions.wizard.WizardModel;
import org.apache.wicket.extensions.wizard.WizardStep;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.CheckBoxMultipleChoice;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.LineGroupDTO;
import cz.sa.shiftPlanner.entities.SymbolVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.pcomponent.EnumCheckBoxMultipleChoice;
import cz.sa.shiftPlanner.services.BusTimetableService;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.utils.MySimpleHoliday;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.BusTurnsPanel;
import cz.sa.shiftPlanner.web.panels.CalendarPanel;

public class TimetableTurnsWizard extends Wizard
{
	@SpringBean(name = "busTimetableService")
	BusTimetableService busTimetableService;
	
	@SpringBean(name = "lineGroupService")
	 LineGroupService lineService;
	
	private IModel<ArrayList<WeekDay>> daysModel;
	private IModel<Date> dateModelFrom = new Model<Date>();
	private IModel<Date> dateModelTo = new Model<Date>();
	private  List<BusTurnsPanel> panels= new ArrayList<BusTurnsPanel>();
	private IModel<ArrayList<Integer>> weekModel;
	private Model<Boolean> saveAll = Model.of(Boolean.FALSE);
	private LineGroupDTO lineGroup;
	private PageParameters parameters;
	
	 public TimetableTurnsWizard(String id, PageParameters parameters){
	        super(id);

	        lineGroup = lineService.get(parameters.get("id").toLong());
	        daysModel = new Model<ArrayList<WeekDay>>(new ArrayList<WeekDay>());
	        weekModel = new Model<ArrayList<Integer>>();
	        this.parameters = parameters;
	        ArrayList<Integer> def = new ArrayList<Integer>();
	        def.add(1);
	        weekModel.setObject(def);
	        
	        WizardModel model = new WizardModel();
	        model.add(new Days(daysModel, weekModel, dateModelFrom, dateModelTo));
	        model.add(new Turns(daysModel, weekModel, dateModelFrom, dateModelTo));

	        // initialize the wizard
	        init(model);
	    }
	 
    private final class Days extends WizardStep
    {
    	private List<WeekDay> choices = Arrays.asList(WeekDay.values());
    	private List<Integer> weekChoices = new ArrayList<Integer>(Arrays.asList(1,2,3,4));
    	
        public Days(final IModel<ArrayList<WeekDay>> daysModel, final IModel<ArrayList<Integer>> weekModel, final IModel<Date> dateModelFrom, final IModel<Date> dateModelTo){
        	super("","Tocky pro linku: "+lineGroup.getName());
        	
        	add(new FeedbackPanel("feedback"));
        	
        	Form weekForm = new Form("weekForm");
            
        	if(lineGroup.isInterstate()){
        		weekChoices.remove(new Integer(3));
        		weekChoices.remove(new Integer(4));
        	}
    		weekForm.add(new CheckBoxMultipleChoice<Integer>("dayTypes", weekModel, weekChoices));
    		
    		add(weekForm);
    		
            Form form = new Form("editForm");
            
    		form.add(new EnumCheckBoxMultipleChoice<WeekDay>("dayTypes", daysModel, choices));
    		add(form);
    	
			CalendarPanel calendar = new CalendarPanel("calendar",
					dateModelFrom);
			add(calendar);

			CalendarPanel calendar2 = new CalendarPanel("calendar2",
					dateModelTo);
			add(calendar2);
        }
        
    }

    private final class Turns extends WizardStep {
        
    	@SpringBean(name = "busTimetableService")
    	BusTimetableService busTimetableService;
    	
    	private IModel<Date> dateModelFrom;
		private IModel<Date> dateModelTo;
		private IModel<ArrayList<WeekDay>> daysModel;
		
		private IModel<ArrayList<Integer>> weekModel;

    	public Turns(final IModel<ArrayList<WeekDay>> daysModel, final IModel<ArrayList<Integer>> weekModel, IModel<Date> dateModelFrom, IModel<Date> dateModelTo){
    	  super("","Tocky pro linku: "+ lineGroup.getName());
	    	this.dateModelFrom = dateModelFrom;
			this.dateModelTo = dateModelTo;
			this.daysModel = daysModel;
			this.weekModel = weekModel;
    	}
    	
		@Override
		protected void onBeforeRender() {
			super.onBeforeRender();
	
			createPanels(this.daysModel, this.weekModel, this.dateModelFrom, this.dateModelTo);
		}
    	
		
        private void createPanels(final IModel<ArrayList<WeekDay>> daysModel, final IModel<ArrayList<Integer>> weekModel, IModel<Date> dateModelFrom, IModel<Date> dateModelTo){
        	
        	CheckBox weeks = new CheckBox("saveAll", saveAll);
        	Label saveAllLabel = new Label("saveAllLabel", new ResourceModel("saveAll.label"));
        	if(!lineGroup.isInterstate()){
        	  weeks.setVisible(false);
        	  saveAllLabel = new Label("saveAllLabel", "");
        	}
        	
        	//pokud nejsou vybrany v multipleChoice zadne dny pak nastava else vetev kde se pouzije datum z casoveho useku pro editaci vyjimek
        	if(daysModel.getObject().size() != 0){
	    		ListView<WeekDay> listview = new ListView<WeekDay>("listview", daysModel) {
	    		    protected void populateItem(final ListItem<WeekDay> item) {
	    		        IModel<List< List<BusTimetableDTO>>> model = new LoadableDetachableModel<List<List<BusTimetableDTO>>>() {
	    	    			@Override
	    	    			protected  List<List<BusTimetableDTO>> load(){
	    	    				//TODO change validity only to validity from session. new DateTime() is easy way to developing
	    	    				return Lists.newArrayList(busTimetableService.findByLineGroupIdAndDay(
	    	    				    lineGroup.getId(),
	    	    						item.getModelObject(),
	    	    						PlannerSession.get().getSelectedValidity()
	    	    						).values());
	    	    			}
	    	    		};
	    	    		
	    	    		//label o jaky den se jedna
	    	    		item.add(new Label("day", new ResourceModel("day."+item.getModelObject().name())));
	    	    		item.add(new Label("week",  " - tyden " + weekModel.getObject().get(0)));
	      				item.add(new LineViews("times", model, item.getModelObject(), weekModel.getObject().get(0)));
	    		    }
	    		};
	    		addOrReplace(listview);
	    		addOrReplace(weeks, saveAllLabel);
        	}else{
        		DateTime from = new DateTime(dateModelFrom.getObject());
        		DateTime to = new DateTime(dateModelTo.getObject());
        		List<DateTime> times = new ArrayList<DateTime>();
        		while (from.compareTo(to) <= 0) {
        			times.add(new DateTime(from));
        			from = from.plusDays(1);
        		}
        		ListView<DateTime> listview = new ListView<DateTime>("listview", times) {
	    		    protected void populateItem(final ListItem<DateTime> item) {
	    		        IModel<List< List<BusTimetableDTO>>> model = new LoadableDetachableModel<List<List<BusTimetableDTO>>>() {
	    	    			@Override
	    	    			protected  List<List<BusTimetableDTO>> load(){
	    	    				//TODO change validity only to validity from session. new DateTime() is easy way to developing
	    	    				return Lists.newArrayList(busTimetableService.findByLineGroupIdAndDay(
	    	    				    lineGroup.getId(),
	    	    						item.getModelObject(), 
	    	    						PlannerSession.get().getSelectedValidity()
	    	    						).values());
	    	    			}
	    	    		};
	    	    		
	    	    		item.add(new Label("day", dateFormat(item.getModelObject())));
	    	    		item.add(new Label("week",  ""));
	      				item.add(new LineViews("times",
					      						model,
					      						WeekDay.valueOf(item.getModelObject().dayOfWeek().getAsText(Locale.ENGLISH).toUpperCase()),
					      						new DateTime(item.getModelObject().getYear(), item.getModelObject().getMonthOfYear(), item.getModelObject().getDayOfMonth(), 0, 0, 0, 0)));
	    		    }
	    		};
	    		addOrReplace(listview);
	    		weeks.setVisible(false);
	    		addOrReplace(weeks);
        	}
    		
        }
        
        
        private class LineViews extends ListView<List<BusTimetableDTO>>{
    		
        	private WeekDay day;
        	private DateTime validity = null;
        	private Integer week = null;
    		
        	public LineViews(String id, IModel<List< List<BusTimetableDTO>>> model, WeekDay day, Integer week) {
    		      super(id, model);
    		      this.day = day;
    		      this.week = week;
      		}
        	
    		public LineViews(String id, IModel<List< List<BusTimetableDTO>>> model, WeekDay day, DateTime validity) {
  		      super(id, model);
  		      this.day = day;
  		      this.validity = validity;
    		}

    		@Override
    		protected void populateItem(ListItem <List<BusTimetableDTO>> item) {
    			BusTurnsPanel panel = new BusTurnsPanel("lines",item.getModel(), day, week, validity);
    			panels.add(panel);
    			item.add(panel);
    			
    		}
    	}
        
        private String dateFormat(DateTime time){
        	String sTime;
        	sTime= time.getDayOfMonth() + "." + time.getMonthOfYear() + "." + time.getYear();
        	return sTime;
        }
    }


    @Override
    public void onCancel()
    {
        setResponsePage(WizardPage.class,parameters);
    }
    
   @Override
   public void onActiveStepChanged(IWizardStep newStep) {
	  panels = new ArrayList<BusTurnsPanel>();
	  super.onActiveStepChanged(newStep);
   }

   
    @Override
    public void onFinish()
    {
    	for(BusTurnsPanel p : panels){
    		p.save(saveAll.getObject());
    	}
        setResponsePage(WizardPage.class,parameters);
    }

    
}
